/*
 * Copyright (c) 2018 Honeywell International, Inc. All rights reserved.
 *  This file contains trade secrets of Honeywell International, Inc. No part may be reproduced or transmitted in any form by any means or for any purpose
 *  without the express written permission of Honeywell.
 */

/* globals console    */ // console ignore:line
/* globals setInterval    */ // setInterval ignore:line
/* globals clearInterval    */ // clearInterval ignore:line
/* globals navigator */ // jshint ignore:line

/**
 * this is a copy from the schedule.js with extensions: save to controller, read From Controller, and the refresh
 * @private
 * @module nmodule/honBACnetUtilities/rc/HonBacnetScheduler
 */
define(['baja!',
        'lex!baja',
        'jquery',
        'Promise',
        'underscore',
        'bajaux/events',
        'bajaux/mixin/responsiveMixIn',
        'bajaux/Properties',
        'bajaux/mixin/subscriberMixIn',
        'bajaux/util/SaveCommand',
        'nmodule/js/rc/switchboard/switchboard',
        'nmodule/webEditors/rc/fe/baja/TabbedCompositeEditor',
        'nmodule/webEditors/rc/fe/baja/util/DepthSubscriber',
        'nmodule/webEditors/rc/fe/config/CompositeBuilder',
        'nmodule/schedule/rc/baja/model/ScheduleTranslator',
        'nmodule/schedule/rc/baja/fe/BajaWeekEditor',
        'nmodule/schedule/rc/fe/WeekEditor',
        'nmodule/schedule/rc/fe/ScheduleSummaryView',
        'nmodule/schedule/rc/fe/SpecialEventsEditor',
        'nmodule/schedule/rc/fe/menu/DayScheduleContextMenu',
        'nmodule/schedule/rc/fe/menu/WeekScheduleContextMenu',
        'nmodule/schedule/rc/servlets/ssc',
        'nmodule/schedule/rc/util/scheduleUtils',
        'css!nmodule/schedule/rc/fe/schedule'], function (baja,
                                                          lexs,
                                                          $,
                                                          Promise,
                                                          _,
                                                          events,
                                                          responsiveMixIn,
                                                          Properties,
                                                          subscribable,
                                                          SaveCommand,
                                                          switchboard,
                                                          TabbedCompositeEditor,
                                                          DepthSubscriber,
                                                          CompositeBuilder,
                                                          ScheduleTranslator,
                                                          BajaWeekEditor,
                                                          WeekEditor,
                                                          ScheduleSummaryView,
                                                          SpecialEventsEditor,
                                                          DayScheduleContextMenu,
                                                          WeekScheduleContextMenu,
                                                          ssc,
                                                          scheduleUtils) {

  'use strict';

  var bajaLex = lexs[0],
      MODIFY_EVENT = events.MODIFY_EVENT,
      _schedule = null,
      _dom = null,
      orgArguments = null,
      refreshInterval = 300000, //by default 5 min
      _refreshTicket = null,
      isRefreshing = false,
      isSaving = false;


    var tpl = function tpl(ed) {
        return '\n    <div class=\"ux-tab-bar-container ux-fixed-header\"></div>    \n    <div class=\"ux-tab-panel-container ux-fullscreen-support ux-body ux-tab-content ux-no-scroll\">    \n      <div class=\"week-panel ux-fullscreen ux-tab-inner-content\"></div>\n      <div class=\"specialEvents-panel ux-fullscreen ux-tab-inner-content\"></div>\n      <div class=\"scheduleSummary-panel ux-fullscreen ux-tab-inner-content\"></div>\n    </div>';
    };


    /**
     * HonBacnetScheduler
     *
     * A Tabbed Editor that has three tabs (weekly schedule, special events, summary)
     *
     * @param {Object} params
     * @constructor
     */
    var HonBacnetScheduler = function HonBacnetScheduler(params) {
        var that = this;

        switchboard(this, {
            $reloadSummary: {allow: 'oneAtATime', onRepeat: 'returnLast'}
        });

        TabbedCompositeEditor.call(that, $.extend({}, {
            moduleName: 'honBACnetUtilities',
            keyName: 'HonBacnetScheduler',
        }, params));

    // TODO Test
    that.getCommandGroup().add(new SaveCommand());

    // TODO Test
    subscribable(this);

    this.$subscriber = new DepthSubscriber(7);
    responsiveMixIn(this, {
      'scheduler-sm': {
        maxWidth: 760
      }
    });
  };
  HonBacnetScheduler.prototype = Object.create(TabbedCompositeEditor.prototype);
  HonBacnetScheduler.prototype.constructor = HonBacnetScheduler;

  var getTimeLabels = _.once(function () {
    return Promise.all([3, 6, 9, 12, 15, 18, 21].map(function (hour) {
      return baja.Time.make({ hour: hour }).toTimeString();
    }));
  });

  HonBacnetScheduler.prototype.doInitialize = function (dom) {
    var that = this;
    _dom = dom;
    dom.addClass('Scheduler ux-fullscreen').html(tpl(this));

    // TODO Test
    dom.on(MODIFY_EVENT, '.ux-tab-panel-container', function () {
      that.setModified(true);
      that.$reloadSummaryOnShowTabPanel = true;
      return false;
    });
    _refreshTicket = setInterval(function(){refreshTicketProc(that);}, refreshInterval);
    return TabbedCompositeEditor.prototype.doInitialize.call(that, dom.find('.ux-tab-bar-container'));
  };

  HonBacnetScheduler.prototype.doLoad = function (weeklySchedule) {
    var that = this;
    orgArguments = arguments;
    _schedule = weeklySchedule;
    var navOrd = weeklySchedule.getNavOrd();

    function update(property) {
      var facets;
      if(property.getName() === "refreshInterval") {
             baja.Ord.make("station:|slot:" + _schedule.toPathString()).get().then(function (comp){
               return comp.serverSideCall({
                     typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                     methodName: "getRefreshInterval",
                     value: "",
               });
             }).then(function (response) {
               if(_refreshTicket!== null) {
                 clearInterval(_refreshTicket);
               }
               refreshInterval = response;
               _refreshTicket = setInterval(function(){refreshTicketProc(that);}, refreshInterval);
             });
      }
      else if((property.getName() === "schedule") || (property.getName() === "effective") ||
              (property.getName() === "nextValue"))
      {
        if((!isSaving ) && (!isRefreshing) &&(!that.isLoading())) {
          _dom.off(MODIFY_EVENT, '.ux-tab-panel-container', function () {
                         that.setModified(false);
          });

          $.when(refresh()).done(function() {
            _dom.on(MODIFY_EVENT, '.ux-tab-panel-container', function () {
                            that.setModified(true);
                            that.$reloadSummaryOnShowTabPanel = true;
                            return false;
            });
          });
        }
      }
      else if(property.getName() === "facets")
      {
        facets= _schedule.get('facets');
        that.$snapshot.set({
             slot: 'facets',
             value: facets.newCopy(true),
        });

      }
    }


    return ssc.loadScheduleInfo(navOrd).then(function (ref) {
      var snapshot = ref.snapshot;
      var refBase = ref.refBase;
      var options = ref.options

      that.$snapshot = snapshot;
      that.$refBase = refBase;
      that.$sourceOrd = weeklySchedule.getNavOrd();
      that.$translator = new ScheduleTranslator(snapshot.get("facets"));
      that.$options = options;

      return that.$translator.toModel(snapshot);

    }).then(function (model) {
      that.$model = model;
      var tabbedCompEditor = TabbedCompositeEditor.prototype.doLoad.apply(that, orgArguments);
      return tabbedCompEditor
    }).then(function (editor) {
      // TODO Test
      new WeekScheduleContextMenu(that.$getWeekEditor()).arm();
      new DayScheduleContextMenu(that.$getSpecialEventsEditor().$getDayEditor()).arm();

      var builder = that.getBuilder(),
          weekEd = builder.getEditorFor('week'),
          eventsEd = builder.getEditorFor('specialEvents');
      weekEd.on('defaultOutputChanged', function (value) {
        return eventsEd.properties().add('defaultOutput', value);
      });
      weekEd.on('facetsChanged', function (value) {
        eventsEd.properties().add('facets', value);
        that.$translator.facets(value);
      });


      //attach the changed function to the update function
      that.$subscriber.attach('changed', update);

      // read From Controller
      _schedule.serverSideCall({
            typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
            methodName: "readSchedule",
            value: "", // This is the argument for the Server Side Call
      })
      .catch(function(err) {
            console.log("honBACnetUtilities.readSchedule serverSideCall throws exception " + ", err: " + err);
      });
      if (that.$options.readonly) {
        return that.setReadonly(true);
      }
    });
  };

  /**
   * Show the Tab Panel and if the is a pending reload summary request from a modification, reload the summary view.
   * @param {String} choice
   * @returns {Promise}
   * @private
   */
  HonBacnetScheduler.prototype.$showTabPanel = function (choice) {
    var that = this;
    return Promise.resolve(TabbedCompositeEditor.prototype.$showTabPanel.apply(this, arguments)).then(function () {
      if (choice === "scheduleSummary" && that.$reloadSummaryOnShowTabPanel) {
        that.$reloadSummaryOnShowTabPanel = false;
          return that.$reloadSummary()["catch"](baja.error);
      }
    });

  };

  /**
   * When the HonBacnetScheduler is modified, update the Schedule Summary after waiting
   * for the tab be be shown.
   * @returns {Promise}
   * @private
   */
  HonBacnetScheduler.prototype.$reloadSummary = function () {
    var builder = this.getBuilder(),
        summaryEd = builder.getEditorFor('scheduleSummary');

    return this.read().then(function (value) {
      return summaryEd.load(value);
    });
  };

  HonBacnetScheduler.prototype.makeBuilder = function () {
    var that = this,
        builder = new CompositeBuilder();

    builder.getKeys = function () {
      return ['week', 'specialEvents', 'scheduleSummary'];
    };

    builder.getDomFor = function (key) {
        return that.jq().find('.'.concat(key + '-panel'));
    };

    builder.getValueFor = function (key) {
      switch (key) {
        case 'week':
          return that.$model;
        case 'specialEvents':
          return that.$snapshot.get('schedule').get('specialEvents');
        case 'scheduleSummary':
          return that.$snapshot;
      }
    };

    builder.getTabFor = function (key) {
      switch (key) {
        case 'week':
          return {
            module: 'schedule',
            lex: 'scheduler.weeklySchedule',
          };
        case 'specialEvents':
          return {
            module: 'schedule',
            lex: 'scheduler.specialEvents',
          };
        case 'scheduleSummary':
          return {
            module: 'schedule',
            lex: 'scheduler.scheduleSummary',
          };
      }
    };

    builder.getConfigFor = function (key) {
      var snapshot = that.$snapshot,
          defaultOutput = snapshot.get('defaultOutput'),
          facets = snapshot.get('facets'),
          effectiveValue = scheduleUtils.getNewBlockValue(defaultOutput, facets);
      switch (key) {
        case 'week':
          return getTimeLabels().then(function (timeLabels) {
            return {
              type: BajaWeekEditor,
              properties: Properties.extend(facets.toObject(), {
                effectiveValue: effectiveValue,
                firstDayOfWeek: bajaLex.get('weekday.firstDayOfWeek'),
                snapshot: snapshot,
                timeLabels: timeLabels,
                translator: that.$translator,
                sourceOrd: that.$sourceOrd,
              })
            };
          });
        case 'specialEvents':
          return getTimeLabels().then(function (timeLabels) {
            return {
              type: SpecialEventsEditor,
              properties: {
                  defaultOutput: defaultOutput.newCopy(),
                  effectiveValue: effectiveValue,
                  facets: facets,
                  refBase: that.$refBase,
                  sourceOrd: that.$sourceOrd,
                  includeReference: true,
                  timeLabels: timeLabels,
                  translator: that.$translator
              }
            };
          });
        case 'scheduleSummary':
          return {
            type: ScheduleSummaryView,
            properties: {
                facets: facets,
                refBase: that.$refBase,
                sourceOrd: that.$sourceOrd
            }
          };
      }
    };

    return builder;
  };

  HonBacnetScheduler.prototype.$getPropertiesElement = function () {
    return this.jq().find('.properties');
  };

  HonBacnetScheduler.prototype.$getTabElement = function (key) {
    return this.jq().children('.ux-tab-panel-container').children('.' + key + '-panel');
  };

  HonBacnetScheduler.prototype.$getWeekEditor = function () {
    return this.getBuilder().getEditorFor('week');
  };

  /**
   * Read the WeekSchedule and ignore any validation errors. This allows the
   * Summary Tab to get the current value of all valid changes without bothering
   * the user about any changes that are not yet valid.
   */
  HonBacnetScheduler.prototype.$doReadWeekSchedule = function () {
    var that = this,
        weekEd = that.$getWeekEditor();
    return weekEd.read().then(function (weekSchedule) {
      return that.$translator.fromModel(weekSchedule);
    });
  };

  /**
   * Save the WeekSchedule and report any validation errors.
   */
  HonBacnetScheduler.prototype.$doSaveWeekSchedule = function () {
    var that = this,
        weekEd = that.$getWeekEditor();
      return weekEd.save().then(function () {
          return weekEd.read();
      }).then(function (weekSchedule) {
          return that.$translator.fromModel(weekSchedule);
      });
  };

    HonBacnetScheduler.prototype.$getSpecialEventsEditor = function () {
        return this.getBuilder().getEditorFor('specialEvents');
    };

    HonBacnetScheduler.prototype.$doReadSpecialEvents = function () {
        return this.$getSpecialEventsEditor().read();
    };

    HonBacnetScheduler.prototype.$doSaveSpecialEvents = function () {
        var specialEventsEd = this.$getSpecialEventsEditor();
        return specialEventsEd.save().then(function () {
            return specialEventsEd.read();
        });
    };

    HonBacnetScheduler.prototype.$getScheduleSummaryView = function () {
        return this.getBuilder().getEditorFor('scheduleSummary');
  };

  /**
   * Collect the current state of the Scheduler for any field editors
   * that properly validate, but do not save them. This includes the weekly
   * schedule, special events, and any changes to facets so that the Summary Tab
   * can use this to display the current events as they are being edited.
   * @returns {Promise}
   */
  HonBacnetScheduler.prototype.doRead = function () {
    var that = this;
      return Promise.all([that.$doReadWeekSchedule(), that.$doReadSpecialEvents(), that.$translator.facets()]).spread(function (weekSchedule, specialEvents, facets) {
          that.$snapshot.get('schedule').set({
              slot: 'week',
              value: weekSchedule.newCopy(true),
          });
          that.$snapshot.get('schedule').set({
              slot: 'specialEvents',
              value: specialEvents.newCopy(true),
          });
          that.$snapshot.set({
              slot: 'facets',
            value: facets.newCopy(true),
      });
        return that.$snapshot;
   });
  };


  HonBacnetScheduler.prototype.doSave = function () {
    isSaving = true;
    var that = this;
    // var schedulePromise = Promise.all([
    return Promise.all([that.$doSaveWeekSchedule(), that.$doSaveSpecialEvents(), that.getChildEditors().saveAll()]).spread(
          function (weekSchedule, specialEvents) {
                  console.log("set weeklyschedule ");
      that.$snapshot.get('schedule').set({
        slot: 'week',
        value: weekSchedule.newCopy(true),
      });
      that.$snapshot.get('schedule').set({
        slot: 'specialEvents',
        value: specialEvents.newCopy(true),
      });
      var facets = that.$translator.facets();
       that.$snapshot.set({
                  slot: 'facets',
                  value: facets.newCopy(true),
            });


                  return ssc.save(that.value().getNavOrd(), that.$snapshot);
          }).catch(function(err) {
         console.log("schedulePromise failed " +  err);
          }).then(function(schedule){
              console.log("send serverSideCall for saveSchedule")
        return  _schedule.serverSideCall({
                      typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                      methodName: "saveSchedule",
                      value: schedule,
              }).catch(function(err) {
          console.log("serverSideCall saveSchedule failed " +  err);
                        //return err;
        });
          }).then(function(){
          var facets = that.$snapshot.get('facets');
          _schedule.set({
                           slot: 'facets',
                           value: facets.newCopy(true),
          });
           _schedule.serverSideCall({
                           typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                           methodName: "setFacets",
                           value: facets, // This is the argument for the Server Side Call
           });
          }).then(function(){
      //read the _schedule from Controller to be sure all the session have the updated schedule
        var readPromise =  _schedule.serverSideCall({
                    typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                    methodName: "readSchedule",
                    value: "", // This is the argument for the Server Side Call
        })
        .catch(function(err) {
              console.log("serverSideCall readSchedule failed " +  err);
                                    //return err;
        });
        isSaving = false;
        return readPromise;

          }).catch(function(err) {
          console.log('failed ', err);
          return err;
      });
  };

  HonBacnetScheduler.prototype.doDestroy = function () {
    this.jq().removeClass('ux-fullscreen Scheduler');

    return TabbedCompositeEditor.prototype.doDestroy.apply(this, arguments);
  };

  ///////////LOCAL FUNCTIONS

  function refreshTicketProc(that)
  {
    _schedule.serverSideCall({
                          typeSpec: "honBACnetUtilities:HonScheduleServerSideCallHandler",
                          methodName: "readSchedule",
                          value: "",
    })
    .then(function(response){
      _dom.off(MODIFY_EVENT, '.ux-tab-panel-container', function () {
                                     that.setModified(false);

      });

      $.when(refresh()).done(function() {
            _dom.on(MODIFY_EVENT, '.ux-tab-panel-container', function () {
                    that.setModified(true);
                    that.$reloadSummaryOnShowTabPanel = true;
                    return false;
            });
      });
    });
  }

  function refresh(that)
  {
    isRefreshing = true;
    refreshWeekTab(that);
    isRefreshing = false;
   return Promise.resolve();
 }

  function refreshWeekTab(that)
  {
    try{
      var weekEditors = that.getChildEditors();
      return Promise.all(weekEditors.map(function (bajaWeekEd) {
        if(bajaWeekEd instanceof BajaWeekEditor )
        {
          var weekEditors = bajaWeekEd.getChildEditors();
          Promise.all(weekEditors.map(function (weekEd) {
            if(weekEd instanceof WeekEditor)
            {
              ssc.getSnapshot(_schedule)
              .then(function (snapshot) {
                 that.$snapshot = snapshot;
                 that.$translator = new ScheduleTranslator(_schedule.get('facets'));
                 return that.$translator.toModel(snapshot);
              })
              .then(function (model) {
                 that.$model = model;
                 var daySchedules = that.$model.getDaySchedules();
                 var count = 0;
                 weekEd.$getDayEditors().forEach(function (dayEditor) {
                   dayEditor.$getClearCommand().invoke();
                   var daySchedule = daySchedules[count];
                   dayEditor.load(daySchedule);
                   count ++;
                 });
              });
            }
          }));
        }
    }));
    }catch(e)
    {
      //console.log(" e " + e);
    }
  }

  return HonBacnetScheduler;
});
