/*
 * Copyright (c) 2018 Honeywell International, Inc. All rights reserved.
 *  This file contains trade secrets of Honeywell International, Inc. No part may be reproduced or transmitted in any form by any means or for any purpose
 *  without the express written permission of Honeywell.
 */

define(['baja!',
  'underscore'], function (
    baja,
    _) {
  'use strict';

  var FrozenEnum = baja.FrozenEnum;

  var PropertyIdFacetsArray = [],
      supportedPropertyIdOrdinals = _.range(0, 385).concat([759, 760, 761, 762, 763, 515, 624, 765, 522, 732, 766]);

  var HonBacnetUtils = function () {
    FrozenEnum.apply(this, arguments);
  };

  HonBacnetUtils.prototype = Object.create(FrozenEnum.prototype);
  HonBacnetUtils.prototype.constructor = HonBacnetUtils;

  HonBacnetUtils.make = function () {
    var d = HonBacnetUtils.DEFAULT;
    return d.make.apply(d, arguments);
  };

  HonBacnetUtils.DEFAULT = new HonBacnetUtils();

  HonBacnetUtils.getPropertyIdFacets = function (ordinal) {
    if (!baja.hasType(ordinal, 'baja:Number')) {
      throw new Error('ordinal required');
    }

    if (!_.contains(supportedPropertyIdOrdinals, ordinal)) {
      return null;
    }

    var f = PropertyIdFacetsArray[ordinal];

    if (!f) {
      var tag = HonBacnetUtils.DEFAULT.getRange().getTag(ordinal);
      f = baja.Facets.make({
        range: baja.EnumRange.make({ ordinals: [ ordinal ], tags: [ tag ] })
      });
      PropertyIdFacetsArray[ordinal] = f;
    }

    return f;
  };

  HonBacnetUtils.encodeToString = function (value)
  {
  };

  HonBacnetUtils.decodeFromString = function (value)
  {
  };

  HonBacnetUtils.convertSecondsToHMS = function (s){
    var h = Math.floor(s / 3600); //Get whole hours
    s -= h * 3600;
    var m = Math.floor(s / 60); //Get remaining minutes
    s -= m * 60;
    return h + ":" + (m < 10 ? '0' + m : m) + ":" + (s < 10 ? '0' + s : s); //zero padding on minutes and seconds
  };

//    HonBacnetUtils.escape = function escape(value)
//    {
//    };
//
//    HonBacnetUtils.unescape = function unescape(value)
//    {
//      while(value.includes("$20"))
//      {
//        value = value.replace("$20", " "); //unescape
//      }
//      return value;
//    };

  //static variables (BacnetConst)
  HonBacnetUtils.ACKED_TRANSITIONS = 0;
  HonBacnetUtils.ACK_REQUIRED = 1;
  HonBacnetUtils.ACTION = 2;
  HonBacnetUtils.ACTION_TEXT = 3;
  HonBacnetUtils.ACTIVE_TEXT = 4;
  HonBacnetUtils.ACTIVE_VT_SESSIONS = 5;
  HonBacnetUtils.ALARM_VALUE = 6;
  HonBacnetUtils.ALARM_VALUES = 7;
  HonBacnetUtils.ALL = 8;
  HonBacnetUtils.ALL_WRITES_SUCCESSFUL = 9;
  HonBacnetUtils.APDU_SEGMENT_TIMEOUT = 10;
  HonBacnetUtils.APDU_TIMEOUT = 11;
  HonBacnetUtils.APPLICATION_SOFTWARE_VERSION = 12;
  HonBacnetUtils.ARCHIVE = 13;
  HonBacnetUtils.BIAS = 14;
  HonBacnetUtils.CHANGE_OF_STATE_COUNT = 15;
  HonBacnetUtils.CHANGE_OF_STATE_TIME = 16;
  HonBacnetUtils.NOTIFICATION_CLASS = 17;
  HonBacnetUtils.REMOVED_1 = 18;
  HonBacnetUtils.CONTROLLED_VARIABLE_REFERENCE = 19;
  HonBacnetUtils.CONTROLLED_VARIABLE_UNITS = 20;
  HonBacnetUtils.CONTROLLED_VARIABLE_VALUE = 21;
  HonBacnetUtils.COV_INCREMENT = 22;
  HonBacnetUtils.DATE_LIST = 23;
  HonBacnetUtils.DAYLIGHT_SAVINGS_STATUS = 24;
  HonBacnetUtils.DEADBAND = 25;
  HonBacnetUtils.DERIVATIVE_VARANT = 26;
  HonBacnetUtils.DERIVATIVE_VARANT_UNITS = 27;
  HonBacnetUtils.DESCRIPTION = 28;
  HonBacnetUtils.DESCRIPTION_OF_HALT = 29;
  HonBacnetUtils.DEVICE_ADDRESS_BINDING = 30;
  HonBacnetUtils.DEVICE_TYPE = 31;
  HonBacnetUtils.EFFECTIVE_PERIOD = 32;
  HonBacnetUtils.ELAPSED_ACTIVE_TIME = 33;
  HonBacnetUtils.ERROR_LIMIT = 34;
  HonBacnetUtils.EVENT_ENABLE = 35;
  HonBacnetUtils.EVENT_STATE = 36;
  HonBacnetUtils.EVENT_TYPE = 37;
  HonBacnetUtils.EXCEPTION_SCHEDULE = 38;
  HonBacnetUtils.FAULT_VALUES = 39;
  HonBacnetUtils.FEEDBACK_VALUE = 40;
  HonBacnetUtils.FILE_ACCESS_METHOD = 41;
  HonBacnetUtils.FILE_SIZE = 42;
  HonBacnetUtils.FILE_TYPE = 43;
  HonBacnetUtils.FIRMWARE_REVISION = 44;
  HonBacnetUtils.HIGH_LIMIT = 45;
  HonBacnetUtils.INACTIVE_TEXT = 46;
  HonBacnetUtils.IN_PROCESS = 47;
  HonBacnetUtils.INSTANCE_OF = 48;
  HonBacnetUtils.INTEGRAL_VARANT = 49;
  HonBacnetUtils.INTEGRAL_VARANT_UNITS = 50;
  HonBacnetUtils.ISSUE_CONFIRMED_NOTIFICATIONS = 51;
  HonBacnetUtils.LIMIT_ENABLE = 52;
  HonBacnetUtils.LIST_OF_GROUP_MEMBERS = 53;
  HonBacnetUtils.LIST_OF_OBJECT_PROPERTY_REFERENCES = 54;
  HonBacnetUtils.LIST_OF_SESSION_KEYS = 55;
  HonBacnetUtils.LOCAL_DATE = 56;
  HonBacnetUtils.LOCAL_TIME = 57;
  HonBacnetUtils.LOCATION = 58;
  HonBacnetUtils.LOW_LIMIT = 59;
  HonBacnetUtils.MANIPULATED_VARIABLE_REFERENCE = 60;
  HonBacnetUtils.MAXIMUM_OUTPUT = 61;
  HonBacnetUtils.MAX_APDU_LENGTH_ACCEPTED = 62;
  HonBacnetUtils.MAX_INFO_FRAMES = 63;
  HonBacnetUtils.MAX_MASTER = 64;
  HonBacnetUtils.MAX_PRES_VALUE = 65;
  HonBacnetUtils.MINIMUM_OFF_TIME = 66;
  HonBacnetUtils.MINIMUM_ON_TIME = 67;
  HonBacnetUtils.MINIMUM_OUTPUT = 68;
  HonBacnetUtils.MIN_PRES_VALUE = 69;
  HonBacnetUtils.MODEL_NAME = 70;
  HonBacnetUtils.MODIFICATION_DATE = 71;
  HonBacnetUtils.NOTIFY_TYPE = 72;
  HonBacnetUtils.NUMBER_OF_APDU_RETRIES = 73;
  HonBacnetUtils.NUMBER_OF_STATES = 74;
  HonBacnetUtils.OBJECT_IDENTIFIER = 75;
  HonBacnetUtils.OBJECT_LIST = 76;
  HonBacnetUtils.OBJECT_NAME = 77;
  HonBacnetUtils.OBJECT_PROPERTY_REFERENCE = 78;
  HonBacnetUtils.OBJECT_TYPE = 79;
  HonBacnetUtils.OPTIONAL = 80;
  HonBacnetUtils.OUT_OF_SERVICE = 81;
  HonBacnetUtils.OUTPUT_UNITS = 82;
  HonBacnetUtils.EVENT_PARAMETERS = 83;
  HonBacnetUtils.POLARITY = 84;
  HonBacnetUtils.PRESENT_VALUE = 85;
  HonBacnetUtils.PRIORITY = 86;
  HonBacnetUtils.PRIORITY_ARRAY = 87;
  HonBacnetUtils.PRIORITY_FOR_WRITING = 88;
  HonBacnetUtils.PROCESS_IDENTIFIER = 89;
  HonBacnetUtils.PROGRAM_CHANGE = 90;
  HonBacnetUtils.PROGRAM_LOCATION = 91;
  HonBacnetUtils.PROGRAM_STATE = 92;
  HonBacnetUtils.PROPORTIONAL_VARANT = 93;
  HonBacnetUtils.PROPORTIONAL_VARANT_UNITS = 94;
  HonBacnetUtils.PROTOCOL_CONFORMANCE_CLASS = 95;
  HonBacnetUtils.PROTOCOL_OBJECT_TYPES_SUPPORTED = 96;
  HonBacnetUtils.PROTOCOL_SERVICES_SUPPORTED = 97;
  HonBacnetUtils.PROTOCOL_VERSION = 98;
  HonBacnetUtils.READ_ONLY = 99;
  HonBacnetUtils.REASON_FOR_HALT = 100;
  HonBacnetUtils.RECIPIENT = 101;
  HonBacnetUtils.RECIPIENT_LIST = 102;
  HonBacnetUtils.RELIABILITY = 103;
  HonBacnetUtils.RELINQUISH_DEFAULT = 104;
  HonBacnetUtils.REQUIRED = 105;
  HonBacnetUtils.RESOLUTION = 106;
  HonBacnetUtils.SEGMENTATION_SUPPORTED = 107;
  HonBacnetUtils.SETPOINT = 108;
  HonBacnetUtils.SETPOINT_REFERENCE = 109;
  HonBacnetUtils.STATE_TEXT = 110;
  HonBacnetUtils.STATUS_FLAGS = 111;
  HonBacnetUtils.SYSTEM_STATUS = 112;
  HonBacnetUtils.TIME_DELAY = 113;
  HonBacnetUtils.TIME_OF_ACTIVE_TIME_RESET = 114;
  HonBacnetUtils.TIME_OF_STATE_COUNT_RESET = 115;
  HonBacnetUtils.TIME_SYNCHRONIZATION_RECIPIENTS = 116;
  HonBacnetUtils.UNITS = 117;
  HonBacnetUtils.UPDATE_INTERVAL = 118;
  HonBacnetUtils.UTC_OFFSET = 119;
  HonBacnetUtils.VENDOR_IDENTIFIER = 120;
  HonBacnetUtils.VENDOR_NAME = 121;
  HonBacnetUtils.VT_CLASSES_SUPPORTED = 122;
  HonBacnetUtils.WEEKLY_SCHEDULE = 123;
  HonBacnetUtils.ATTEMPTED_SAMPLES = 124;
  HonBacnetUtils.AVERAGE_VALUE = 125;
  HonBacnetUtils.BUFFER_SIZE = 126;
  HonBacnetUtils.CLIENT_COV_INCREMENT = 127;
  HonBacnetUtils.COV_RESUBSCRIPTION_INTERVAL = 128;
  HonBacnetUtils.CURRENT_NOTIFY_TIME = 129;
  HonBacnetUtils.EVENT_TIME_STAMPS = 130;
  HonBacnetUtils.LOG_BUFFER = 131;
  HonBacnetUtils.LOG_DEVICE_OBJECT_PROPERTY = 132;
  HonBacnetUtils.ENABLE = 133;
  HonBacnetUtils.LOG_INTERVAL = 134;
  HonBacnetUtils.MAXIMUM_VALUE = 135;
  HonBacnetUtils.MINIMUM_VALUE = 136;
  HonBacnetUtils.NOTIFICATION_THRESHOLD = 137;
  HonBacnetUtils.PREVIOUS_NOTIFY_TIME = 138;
  HonBacnetUtils.PROTOCOL_REVISION = 139;
  HonBacnetUtils.RECORDS_SINCE_NOTIFICATION = 140;
  HonBacnetUtils.RECORD_COUNT = 141;
  HonBacnetUtils.START_TIME = 142;
  HonBacnetUtils.STOP_TIME = 143;
  HonBacnetUtils.STOP_WHEN_FULL = 144;
  HonBacnetUtils.TOTAL_RECORD_COUNT = 145;
  HonBacnetUtils.VALID_SAMPLES = 146;
  HonBacnetUtils.WINDOW_INTERVAL = 147;
  HonBacnetUtils.WINDOW_SAMPLES = 148;
  HonBacnetUtils.MAXIMUM_VALUE_TIMESTAMP = 149;
  HonBacnetUtils.MINIMUM_VALUE_TIMESTAMP = 150;
  HonBacnetUtils.VARIANCE_VALUE = 151;
  HonBacnetUtils.ACTIVE_COV_SUBSCRIPTIONS = 152;
  HonBacnetUtils.BACKUP_FAILURE_TIMEOUT = 153;
  HonBacnetUtils.CONFIGURATION_FILES = 154;
  HonBacnetUtils.DATABASE_REVISION = 155;
  HonBacnetUtils.DIRECT_READING = 156;
  HonBacnetUtils.LAST_RESTORE_TIME = 157;
  HonBacnetUtils.MAINTENANCE_REQUIRED = 158;
  HonBacnetUtils.MEMBER_OF = 159;
  HonBacnetUtils.MODE = 160;
  HonBacnetUtils.OPERATION_EXPECTED = 161;
  HonBacnetUtils.SETTING = 162;
  HonBacnetUtils.SILENCED = 163;
  HonBacnetUtils.TRACKING_VALUE = 164;
  HonBacnetUtils.ZONE_MEMBERS = 165;
  HonBacnetUtils.LIFE_SAFETY_ALARM_VALUES = 166;
  HonBacnetUtils.MAX_SEGMENTS_ACCEPTED = 167;
  HonBacnetUtils.PROFILE_NAME = 168;
  HonBacnetUtils.AUTO_SLAVE_DISCOVERY = 169;
  HonBacnetUtils.MANUAL_SLAVE_ADDRESS_BINDING = 170;
  HonBacnetUtils.SLAVE_ADDRESS_BINDING = 171;
  HonBacnetUtils.SLAVE_PROXY_ENABLE = 172;
  HonBacnetUtils.LAST_NOTIFY_RECORD = 173;
  HonBacnetUtils.SCHEDULE_DEFAULT = 174;
  HonBacnetUtils.ACCEPTED_MODES = 175;
  HonBacnetUtils.ADJUST_VALUE = 176;
  HonBacnetUtils.COUNT = 177;
  HonBacnetUtils.COUNT_BEFORE_CHANGE = 178;
  HonBacnetUtils.COUNT_CHANGE_TIME = 179;
  HonBacnetUtils.COV_PERIOD = 180;
  HonBacnetUtils.INPUT_REFERENCE = 181;
  HonBacnetUtils.LIMIT_MONITORING_INTERVAL = 182;
  HonBacnetUtils.LOGGING_OBJECT = 183;
  HonBacnetUtils.LOGGING_RECORD = 184;
  HonBacnetUtils.PRESCALE = 185;
  HonBacnetUtils.PULSE_RATE = 186;
  HonBacnetUtils.SCALE = 187;
  HonBacnetUtils.SCALE_FACTOR = 188;
  HonBacnetUtils.UPDATE_TIME = 189;
  HonBacnetUtils.VALUE_BEFORE_CHANGE = 190;
  HonBacnetUtils.VALUE_SET = 191;
  HonBacnetUtils.VALUE_CHANGE_TIME = 192;
  HonBacnetUtils.ALIGN_INTERVALS = 193;
  HonBacnetUtils.INTERVAL_OFFSET = 195;
  HonBacnetUtils.LAST_RESTART_REASON = 196;
  HonBacnetUtils.LOGGING_TYPE = 197;
  HonBacnetUtils.RESTART_NOTIFICATION_RECIPIENTS = 202;
  HonBacnetUtils.TIME_OF_DEVICE_RESTART = 203;
  HonBacnetUtils.TIME_SYNCHRONIZATION_INTERVAL = 204;
  HonBacnetUtils.TRIGGER = 205;
  HonBacnetUtils.UTC_TIME_SYNCHRONIZATION_RECIPIENTS = 206;
  HonBacnetUtils.NODE_SUBTYPE = 207;
  HonBacnetUtils.NODE_TYPE = 208;
  HonBacnetUtils.STRUCTURED_OBJECT_LIST = 209;
  HonBacnetUtils.SUBORDINATE_ANNOTATIONS = 210;
  HonBacnetUtils.SUBORDINATE_LIST = 211;
  HonBacnetUtils.ACTUAL_SHED_LEVEL = 212;
  HonBacnetUtils.DUTY_WINDOW = 213;
  HonBacnetUtils.EXPECTED_SHED_LEVEL = 214;
  HonBacnetUtils.FULL_DUTY_BASELINE = 215;
  HonBacnetUtils.REQUESTED_SHED_LEVEL = 218;
  HonBacnetUtils.SHED_DURATION = 219;
  HonBacnetUtils.SHED_LEVEL_DESCRIPTIONS = 220;
  HonBacnetUtils.SHED_LEVELS = 221;
  HonBacnetUtils.STATE_DESCRIPTION = 222;
  HonBacnetUtils.DOOR_ALARM_STATE = 226;
  HonBacnetUtils.DOOR_EXTENDED_PULSE_TIME = 227;
  HonBacnetUtils.DOOR_MEMBERS = 228;
  HonBacnetUtils.DOOR_OPEN_TOO_LONG_TIME = 229;
  HonBacnetUtils.DOOR_PULSE_TIME = 230;
  HonBacnetUtils.DOOR_STATUS = 231;
  HonBacnetUtils.DOOR_UNLOCK_DELAY_TIME = 232;
  HonBacnetUtils.LOCK_STATUS = 233;
  HonBacnetUtils.MASKED_ALARM_VALUES = 234;
  HonBacnetUtils.SECURED_STATUS = 235;
  HonBacnetUtils.ABSENTEE_LIMIT = 244;
  HonBacnetUtils.ACCESS_ALARM_EVENTS = 245;
  HonBacnetUtils.ACCESS_DOORS = 246;
  HonBacnetUtils.ACCESS_EVENT = 247;
  HonBacnetUtils.ACCESS_EVENT_AUTHENTICATION_FACTOR = 248;
  HonBacnetUtils.ACCESS_EVENT_CREDENTIAL = 249;
  HonBacnetUtils.ACCESS_EVENT_TIME = 250;
  HonBacnetUtils.ACCESS_TRANSACTION_EVENTS = 251;
  HonBacnetUtils.ACCOMPANIMENT = 252;
  HonBacnetUtils.ACCOMPANIMENT_TIME = 253;
  HonBacnetUtils.ACTIVATION_TIME = 254;
  HonBacnetUtils.ACTIVE_AUTHENTICATION_POLICY = 255;
  HonBacnetUtils.ASSIGNED_ACCESS_RIGHTS = 256;
  HonBacnetUtils.AUTHENTICATION_FACTORS = 257;
  HonBacnetUtils.AUTHENTICATION_POLICY_LIST = 258;
  HonBacnetUtils.AUTHENTICATION_POLICY_NAMES = 259;
  HonBacnetUtils.AUTHENTICATION_STATUS = 260;
  HonBacnetUtils.AUTHORIZATION_MODE = 261;
  HonBacnetUtils.BELONGS_TO = 262;
  HonBacnetUtils.CREDENTIAL_DISABLE = 263;
  HonBacnetUtils.CREDENTIAL_STATUS = 264;
  HonBacnetUtils.CREDENTIALS = 265;
  HonBacnetUtils.CREDENTIALS_IN_ZONE = 266;
  HonBacnetUtils.DAYS_REMAINING = 267;
  HonBacnetUtils.ENTRY_POINTS = 268;
  HonBacnetUtils.EXIT_POINTS = 269;
  HonBacnetUtils.EXPIRY_TIME = 270;
  HonBacnetUtils.EXTENDED_TIME_ENABLE = 271;
  HonBacnetUtils.FAILED_ATTEMPT_EVENTS = 272;
  HonBacnetUtils.FAILED_ATTEMPTS = 273;
  HonBacnetUtils.FAILED_ATTEMPTS_TIME = 274;
  HonBacnetUtils.LAST_ACCESS_EVENT = 275;
  HonBacnetUtils.LAST_ACCESS_POINT = 276;
  HonBacnetUtils.LAST_CREDENTIAL_ADDED = 277;
  HonBacnetUtils.LAST_CREDENTIAL_ADDED_TIME = 278;
  HonBacnetUtils.LAST_CREDENTIAL_REMOVED = 279;
  HonBacnetUtils.LAST_CREDENTIAL_REMOVED_TIME = 280;
  HonBacnetUtils.LAST_USE_TIME = 281;
  HonBacnetUtils.LOCKOUT = 282;
  HonBacnetUtils.LOCKOUT_RELINQUISH_TIME = 283;
  HonBacnetUtils.MASTER_EXEMPTION = 284;
  HonBacnetUtils.MAX_FAILED_ATTEMPTS = 285;
  HonBacnetUtils.MEMBERS = 286;
  HonBacnetUtils.MUSTER_POINT = 287;
  HonBacnetUtils.NEGATIVE_ACCESS_RULES = 288;
  HonBacnetUtils.NUMBER_OF_AUTHENTICATION_POLICIES = 289;
  HonBacnetUtils.OCCUPANCY_COUNT = 290;
  HonBacnetUtils.OCCUPANCY_COUNT_ADJUST = 291;
  HonBacnetUtils.OCCUPANCY_COUNT_ENABLE = 292;
  HonBacnetUtils.OCCUPANCY_EXEMPTION = 293;
  HonBacnetUtils.OCCUPANCY_LOWER_LIMIT = 294;
  HonBacnetUtils.OCCUPANCY_LOWER_LIMIT_ENFORCED = 295;
  HonBacnetUtils.OCCUPANCY_STATE = 296;
  HonBacnetUtils.OCCUPANCY_UPPER_LIMIT = 297;
  HonBacnetUtils.OCCUPANCY_UPPER_LIMIT_ENFORCED = 298;
  HonBacnetUtils.PASSBACK_EXEMPTION = 299;
  HonBacnetUtils.PASSBACK_MODE = 300;
  HonBacnetUtils.PASSBACK_TIMEOUT = 301;
  HonBacnetUtils.POSITIVE_ACCESS_RULES = 302;
  HonBacnetUtils.REASON_FOR_DISABLE = 303;
  HonBacnetUtils.SUPPORTED_FORMATS = 304;
  HonBacnetUtils.SUPPORTED_FORMAT_CLASSES = 305;
  HonBacnetUtils.THREAT_AUTHORITY = 306;
  HonBacnetUtils.THREAT_LEVEL = 307;
  HonBacnetUtils.TRACE_FLAG = 308;
  HonBacnetUtils.TRANSACTION_NOTIFICATION_CLASS = 309;
  HonBacnetUtils.USER_EXTERNAL_IDENTIFIER = 310;
  HonBacnetUtils.USER_INFORMATION_REFERENCE = 311;
  HonBacnetUtils.USER_NAME = 317;
  HonBacnetUtils.USER_TYPE = 318;
  HonBacnetUtils.USES_REMAINING = 319;
  HonBacnetUtils.ZONE_FROM = 320;
  HonBacnetUtils.ZONE_TO = 321;
  HonBacnetUtils.ACCESS_EVENT_TAG = 322;
  HonBacnetUtils.GLOBAL_IDENTIFIER = 323;
  HonBacnetUtils.VERIFICATION_TIME = 326;
  HonBacnetUtils.BACKUP_AND_RESTORE_STATE = 338;
  HonBacnetUtils.BACKUP_PREPARATION_TIME = 339;
  HonBacnetUtils.RESTORE_COMPLETION_TIME = 340;
  HonBacnetUtils.RESTORE_PREPARATION_TIME = 341;
  HonBacnetUtils.GROUP_MEMBERS = 345;
  HonBacnetUtils.GROUP_MEMBER_NAMES = 346;
  HonBacnetUtils.MEMBER_STATUS_DLAGS = 347;
  HonBacnetUtils.REQUESTED_UPDATE_INTERVAL = 348;
  HonBacnetUtils.COVU_PERIOD = 349;
  HonBacnetUtils.COVU_RECIPIENTS = 350;
  HonBacnetUtils.EVENT_MESSAGE_TEXTS = 351;
  HonBacnetUtils.EVENT_MESSAGE_TEXTS_CONFIG = 352;
  HonBacnetUtils.EVENT_DETECTION_ENABLE = 353;
  HonBacnetUtils.EVENT_ALGORITHM_INHIBIT = 354;
  HonBacnetUtils.EVENT_ALGORITHM_INHIBIT_REF = 355;
  HonBacnetUtils.TIME_DELAY_NORMAL = 356;
  HonBacnetUtils.RELIABILITY_EVALUATION_INHIBIT = 357;
  HonBacnetUtils.FAULT_PARAMETERS = 358;
  HonBacnetUtils.FAULT_TYPE = 359;
  HonBacnetUtils.LOCAL_FORWARDING_ONLY = 360;
  HonBacnetUtils.PROCESS_IDENTIFIER_FILTER = 361;
  HonBacnetUtils.SUBSCRIBED_RECIPIENTS = 362;
  HonBacnetUtils.AUTHORIZATION_EXEMPTIONS = 364;
  HonBacnetUtils.ALLOW_GROUP_DELAY_INHIBIT = 365;
  HonBacnetUtils.CHANNEL_NUMBER = 366;
  HonBacnetUtils.CONTROL_GROUPS = 367;
  HonBacnetUtils.EXECUTION_DELAY = 368;
  HonBacnetUtils.LAST_PRIORITY = 369;
  HonBacnetUtils.WRITE_STATUS = 370;
  HonBacnetUtils.PROPERTY_LIST = 371;
  HonBacnetUtils.SERIAL_NUMBER = 372;
  HonBacnetUtils.BLINK_WARN_ENABLE = 373;
  HonBacnetUtils.DEFAULT_FADETIME = 374;
  HonBacnetUtils.DEFAULT_RAMPRATE = 375;
  HonBacnetUtils.DEFAULT_STEP_INCREMENT = 376;
  HonBacnetUtils.EGRESS_TIMER = 377;
  HonBacnetUtils.IN_PROGRESS = 378;
  HonBacnetUtils.INSTANTANEOUS_POWER = 379;
  HonBacnetUtils.LIGHTING_COMMAND = 380;
  HonBacnetUtils.LIGHTING_COMMAND_DEFAULT_PRIORITY = 381;
  HonBacnetUtils.MAX_ACTUAL_VALUE = 382;
  HonBacnetUtils.MIN_ACTUAL_VALUE = 383;
  HonBacnetUtils.POWER = 384;
  HonBacnetUtils.TRANSITION = 385;

  //propriatary values
  HonBacnetUtils.SAFETY_POSITION             = 759;
  HonBacnetUtils.SAFETY_VALUE                = 760;
  HonBacnetUtils.SENSOR_OFFSET              = 761;
  HonBacnetUtils.TIME_TO_OPEN               = 762;
  HonBacnetUtils.TIME_TO_CLOSE              = 763;
  HonBacnetUtils.EOH_EOV_OPTIMIZATION_FLAG  = 515;
  HonBacnetUtils.DP_IS_SETPOINT             = 624;
  HonBacnetUtils.TERMINAL_PV                = 765;
  HonBacnetUtils.COLOR_MAP                  = 522;
  HonBacnetUtils.CURRENTLY_ACTIVE_PRIORITY  = 732;
  HonBacnetUtils.DIRECT_REVERSE             = 766;


  return HonBacnetUtils;
});
