require.config({
    paths: {}
});

define(
    [
        'jquery',
        'baja!'
    ],
    function ($, baja) {

        var Utils = {};

        Utils.EnvironmentIsWorkbench = function (series) {
            try {
                // in some places we might not have access to the niagara
                // variable.
                if (typeof (this.niagara) === "undefined") {
                    return false;
                }
                return this.niagara.env.type === "wb";
            } catch (error) {
                baja.error(error);
                return false;
            }
        };

        Utils.encode = function (str) {
            return baja.SlotPath.escape(str);
        };

        Utils.getDomain = function () {
            return window.location.protocol + "//" + window.location.host;
        };

        Utils.getPxSize = function (contents) {
            var h = "0",
                w = "0";
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(contents, "text/xml");

            try {
                var canvasPane = Array.from(xmlDoc.getElementsByTagName("CanvasPane"))[0];
                var viewSize = canvasPane.getAttribute("viewSize");
                w = viewSize.substring(0, viewSize.indexOf(","));
                h = viewSize.substring(viewSize.indexOf(",") + 1);
                return {
                    w: w,
                    h: h
                }
            } catch (error) {
                baja.error("Error Parsing Px File" + error);
            }
        };
       Utils.isWidgetOverlapping= function(currentTop,currentLeft,scrollHeight,scrollWidth,top,left,sHeight,sWidth){
        var vTouch=false,hTouch=false, isUp=false,isLeft=false;
        isUp = top < currentTop;
         isLeft = left <currentLeft;
         if(isUp){
          vTouch = top+sHeight > currentTop;
         }else{
            vTouch = currentTop+scrollHeight>= top;
         }
         if(isLeft){
            hTouch = left+sWidth > currentLeft;
         }else{
            hTouch = currentLeft+scrollWidth > left;
         }

         return hTouch && vTouch;
       };
        Utils.parsePxFile = function (contents) {
            var EasyWidgets = [];
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(contents, "text/xml");

            try {

                var webWidgets = Array.from(xmlDoc.getElementsByTagName("EasyBindingWidget"));
                webWidgets.forEach(function (widget, index) {
                    var attributes = Array.from(widget.attributes);
                    var bindings = Array.from(widget.children);

                    var width, height, x, y, easy = false,
                        layout = false,
                        value = "null",
                        alarm = "null",
                        override = "null",
                        showValue = "true",
                        showalarm = "true",
                        showoverride = "true",
                        labelpos = "bottom",
                        data = "{}",
                        labelonly = false,
						widgetName = "",
                        widgetPath = "";

                    var widgetdata = {
                        width: 0,
                        height: 0,
                        x: 0,
                        y: 0,
                        index: -1,
                        value: "null",
                        alarm: "null",
                        override: "null",
                        showValue: "true",
                        showalarm: "true",
                        showoverride: "true",
                        labelpos: "bottom",
                        data: "{}",
                        labelonly: false,
                        widgetName: "",
                        widgetPath: ""
                    };
                    data ={
                            "bindingType":   "generic",
                            "falseImage": "",
                            "trueImage": ""
                        };
                    attributes.forEach(function (attr) {
                        if (attr.name === "valueOnImage") {
                            easy = true;
                            data.trueImage= attr.value;
                        }
                        if(attr.name === "valueOffImage"){
                            data.falseImage= attr.value;
                        }
                        if (attr.name === "layout") {
                            layout = true;
                            var layoutval = attr.value.split(',');
                            y = layoutval[0];
                            x = layoutval[1];
                            width = layoutval[2];
                            height = layoutval[3];
                        }

                        if(attr.name === "labelPosition"){
                            labelpos = attr.value;
                        }
                        if(attr.name === "showValue"){
                            showValue = attr.value;
                        }
                        if(attr.name === "showAlarm"){
                            showalarm = attr.value;
                        }
                        if(attr.name === "showOverride"){
                            showoverride = attr.value;
                        }
                        if(attr.name === "labelOnly"){
                            labelonly = attr.value;
                        }
                        if(attr.name === "widgetName"){
                            widgetName = attr.value;
                        }
                        if(attr.name === "widgetPath"){
                            widgetPath = attr.value;
                        }
                    });

                    bindings.forEach(function (binding) {
                        var bindingAttrs = Array.from(binding.attributes);
                        bindingAttrs.forEach(function (bindingAttr) {
                            if (binding.localName === "EasyAlarmBinding") {
                                if (bindingAttr.name === "ord") {
                                    alarm = bindingAttr.value;
                                }
                            } else if (binding.localName === "EasyOverrideBinding") {
                                if (bindingAttr.name === "ord") {
                                    override = bindingAttr.value;
                                }
                            } else if (binding.localName === "EasyValueBinding") {
                                if (bindingAttr.name === "ord") {
                                    value = bindingAttr.value;
                                }
                            }
                        });

                        Array.from(binding.children).forEach(function (child){
                            var childs = Array.from(child.children);
                            childs.forEach(function (_children){
                                var attributes = Array.from(_children.attributes);

                                if(_children.nodeName !== "EasyValueBinding") {
                                    if (attributes[0].nodeValue === "alarm") {
                                        widgetdata.alarmImage = attributes[1].value;
                                    }
                                    if (attributes[0].nodeValue === "overridden") {
                                        widgetdata.overrideImage = attributes[1].value;
                                    }
                                }
                            })
                        });

                    });

                    if (easy && layout) {
                        widgetdata.width = width;
                        widgetdata.height = height;
                        widgetdata.x = x;
                        widgetdata.y = y;
                        widgetdata.index = index;
                        widgetdata.value = value;
                        widgetdata.alarm = alarm;
                        widgetdata.override = override;
                        widgetdata.showValue = showValue;
                        widgetdata.showalarm = showalarm;
                        widgetdata.showoverride = showoverride;
                        widgetdata.labelpos = labelpos;
                        widgetdata.data = data;
                        widgetdata.labelonly = labelonly;
                        widgetdata.widgetName=widgetName;
                        widgetdata.widgetPath=widgetPath;
                        EasyWidgets.push(widgetdata);
                    }

                });
            } catch (error) {
                baja.error("Error Parsing Px File" + error);
            }

            return EasyWidgets;
        };

        Utils.addLabelToPxFile = function (index, data, pxContents){
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(pxContents, "text/xml");
            var canvaspane = Array.from(xmlDoc.getElementsByTagName("CanvasPane"))[0];

            var ord = data.ord;

            try{
                var newNode = xmlDoc.createElement("EasyBindingWidget");

                // set layout
                var nodeLayoutAttribute;
                nodeLayoutAttribute = xmlDoc.createAttribute("layout");
                nodeLayoutAttribute.value = data.y + "," + data.x + ",250.0,40.0";
                newNode.attributes.setNamedItem(nodeLayoutAttribute);

                newNode.setAttribute("labelOnly", true);
                newNode.setAttribute("valueOnImage", "");
                newNode.setAttribute("valueOffImage", "");
                newNode.setAttribute("labelPosition", "bottom");
                newNode.setAttribute("showValue", true);
                newNode.setAttribute("showAlarm", false);
                newNode.setAttribute("showOverride", false);

                var ebcanvaspane;
                ebcanvaspane = xmlDoc.createElement("EasyBindingCanvasPane");

                var valueImageLabel;
                valueImageLabel = xmlDoc.createElement("Picture");
                valueImageLabel.setAttribute("name", "valueImageLabel");
                valueImageLabel.setAttribute("image", "");

                var alarmImageLabel;
                alarmImageLabel = xmlDoc.createElement("Picture");
                alarmImageLabel.setAttribute("name", "alarmImageLabel");
                alarmImageLabel.setAttribute("image", "");
                alarmImageLabel.setAttribute("ord", ord);
                alarmImageLabel.setAttribute("f", "3");

                var overrideImageLabel;
                overrideImageLabel = xmlDoc.createElement("Picture");
                overrideImageLabel.setAttribute("name", "overrideImageLabel");
                overrideImageLabel.setAttribute("image", "");
                overrideImageLabel.setAttribute("ord", ord);
                overrideImageLabel.setAttribute("f", "3");

                var textOverlayLabel;
                textOverlayLabel = xmlDoc.createElement("BoundLabel");
                textOverlayLabel.setAttribute("name", "textOverlayLabel");
                textOverlayLabel.setAttribute("f", "3");
                var valueBinding;
                valueBinding = xmlDoc.createElement("EasyValueBinding");
                valueBinding.setAttribute("ord", ord);
                var objectToString = xmlDoc.createElement("ObjectToString");
                objectToString.setAttribute("name", "text");
                objectToString.setAttribute("format", "%out.value%")
                valueBinding.appendChild(objectToString);
                textOverlayLabel.appendChild(valueBinding)


                var contextMenuLabel;
                contextMenuLabel = xmlDoc.createElement("Label");
                contextMenuLabel.setAttribute("name", "contextMenuLabel");
                contextMenuLabel.setAttribute("image", "");
                contextMenuLabel.setAttribute("f", "3");
                var contextValueBinding;
                contextValueBinding = xmlDoc.createElement("EasyValueBinding");
                contextValueBinding.setAttribute("ord", ord);
                contextMenuLabel.appendChild(contextValueBinding)


                ebcanvaspane.appendChild(valueImageLabel);
                ebcanvaspane.appendChild(alarmImageLabel);
                ebcanvaspane.appendChild(overrideImageLabel);
                ebcanvaspane.appendChild(textOverlayLabel);
                ebcanvaspane.appendChild(contextMenuLabel);
                newNode.appendChild(ebcanvaspane);


                var ebvalueOrdAttribute;
                ebvalueOrdAttribute = xmlDoc.createElement("EasyValueBinding");
                ebvalueOrdAttribute.setAttribute("name", "valueBinding");
                ebvalueOrdAttribute.setAttribute("ord", "station:|slot:/"+data.ord);

                var ebvalueImageAtrribute;
                ebvalueImageAtrribute = xmlDoc.createElement("IEbConverter");
                ebvalueImageAtrribute.setAttribute("name", "valueImage");

                ebvalueOrdAttribute.appendChild(ebvalueImageAtrribute);
                newNode.appendChild(ebvalueOrdAttribute);

                var ebalarmOrdAttribute;
                ebalarmOrdAttribute = xmlDoc.createElement("EasyAlarmBinding");
                ebalarmOrdAttribute.setAttribute("name", "alarmBinding");
                ebalarmOrdAttribute.setAttribute("ord", "null");

                newNode.appendChild(ebalarmOrdAttribute);

                var eboverrideOrdAttribute;
                eboverrideOrdAttribute = xmlDoc.createElement("EasyOverrideBinding");
                eboverrideOrdAttribute.setAttribute("name", "overrideBinding");
                eboverrideOrdAttribute.setAttribute("ord", "null");

                newNode.appendChild(eboverrideOrdAttribute);

                // update the canvas pane with our new widget
                canvaspane.appendChild(newNode);
            }catch (error) {
                baja.error("Error Parsing Px File" + error);
            }

            return (new XMLSerializer().serializeToString(xmlDoc));
        };

        Utils.updateLabelInPxFile = function (updatedIndex, data, pxContents){
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(pxContents, "text/xml");

            try{

                var webWidgets = Array.from(xmlDoc.getElementsByTagName("EasyBindingWidget"));
                webWidgets.forEach(function (widget, index) {
                    if (index.toString() === updatedIndex) {
                        var attributes = Array.from(widget.attributes);

                        attributes.forEach(function (attr){
                            if (attr.name === "layout"){
                                attr.value = data.y + "," + data.x + ",250.0,40.0";
                            }
                        });
                    }
                });
            }catch (error) {
                baja.error("Error Parsing Px File" + error);
            }

            return (new XMLSerializer().serializeToString(xmlDoc));
        };

        Utils.updatePxFile = function (updatedIndex, data, pxContents) {
            // console.log("updatePxFile", pxContents);
            if(pxContents === undefined || pxContents === null){
                return;
            }
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(pxContents, "text/xml");
            var imports = Array.from(xmlDoc.getElementsByTagName("import"))[0];
            var importChildren = Array.from(imports.children);

            try {

                var kitPxFound = false;
                importChildren.forEach(function(mod) {
                    var attributes = Array.from(mod.attributes);
                    attributes.forEach(function (attr){
                        if (attr.name === "name"){
                            if (attr.value === "kitPx"){
                                kitPxFound = true;
                            }
                        }
                    });
                });

                if (!kitPxFound){
                    var newImport = xmlDoc.createElement("module");
                    newImport.setAttribute("name", "kitPx");
                    imports.appendChild(newImport);
                }

                var webWidgets = Array.from(xmlDoc.getElementsByTagName("EasyBindingWidget"));
                webWidgets.forEach(function (widget, index) {
                    if (index === updatedIndex) {
                        var bindings = Array.from(widget.children).filter(function(ele){return ele.localname = "EasyValueBinding"}).concat(Array.from(widget.getElementsByTagName("EasyValueBinding")));
                        var attributes = Array.from(widget.attributes);

                        var string = ["labelPosition", "showValue", "showAlarm", "showOverride"];

                        string.forEach(function (string){
                            if(widget.hasAttribute(string)){
                                return;
                            }
                            else{
                                switch (string){
                                    case "labelPosition":
                                        widget.setAttribute("labelPosition", "bottom");
                                        break;
                                    case "showValue":
                                        widget.setAttribute("showValue", true);
                                        break;
                                    case "showAlarm":
                                        widget.setAttribute("showAlarm", true);
                                        break;
                                    case "showOverride":
                                        widget.setAttribute("showOverride", true);
                                        break;

                                }
                            }
                        });

                        attributes.forEach(function (attr) {

                            if(attr.name === "labelPosition"){
                                attr.value = data.labelposition;
                            }
                            if(attr.name === "showValue"){
                                attr.value  = data.showValue;
                            }
                            if(attr.name === "showAlarm"){
                                attr.value  = data.showalarm;
                            }
                            if(attr.name === "showOverride"){
                                attr.value  = data.showoverride;
                            }
                        });

                        bindings.forEach(function (binding) {
                            var bindingAttrs = Array.from(binding.attributes);
                            var foundOrd = false;
                            var bindingType = binding.localName;

                            bindingAttrs.forEach(function (bindingAttr) {
                                if (binding.localName === "EasyAlarmBinding") {
                                    if (bindingAttr.name === "ord") {
                                        bindingAttr.value = data.alarm;
                                        foundOrd = true;
                                    }
                                } else if (binding.localName === "EasyOverrideBinding") {
                                    if (bindingAttr.name === "ord") {
                                        bindingAttr.value = data.override;
                                        foundOrd = true;
                                    }
                                } else if (binding.localName === "EasyValueBinding") {
                                    if (bindingAttr.name === "ord") {
                                        bindingAttr.value = data.value;
                                        foundOrd = true;
                                    }
                                }
                            });
                            if (!foundOrd) {
                                //add ord attr
                                var nodeOrdAttribute;
                                nodeOrdAttribute = xmlDoc.createAttribute("ord");

                                switch (bindingType) {
                                    case "EasyAlarmBinding":
                                        nodeOrdAttribute.value = data.alarm;
                                        break;
                                    case "EasyOverrideBinding":
                                        nodeOrdAttribute.value = data.override;
                                        break;
                                    case "EasyValueBinding":
                                        nodeOrdAttribute.value = data.value;
                                        break;
                                }

                                binding.attributes.setNamedItem(nodeOrdAttribute);

                            }
                        });

                    }
                });
            } catch (error) {
                baja.error("Error Parsing Px File" + error);
            }

            return (new XMLSerializer().serializeToString(xmlDoc));
        };


        return Utils;
    }
);