/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225
 *
 */

/**
 * A module defining `EasyDatabaseManagerWidget`.
 *
 * @module nmodule/easyDatabaseManager/rc/js/EasyDatabaseManagerWidget
 */
define(['baja!',
        'bajaux/Widget',
        'bajaux/mixin/subscriberMixIn',
        'jquery',
        'Promise',
        'nmodule/webEditors/rc/fe/baja/util/DepthSubscriber',
        'nmodule/easyDatabaseManager/rc/js/licenseStatusView',
        'nmodule/easyDatabaseManager/rc/js/unusedPointsView',
        'nmodule/easyDatabaseManager/rc/js/unwantedPointsView',
        'nmodule/easyDatabaseManager/rc/js/reviewPointsView',
        'nmodule/easyDatabaseManager/rc/js/deletePointsView',
        'nmodule/easyDatabaseManager/rc/js/postDeleteStatusView',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/PointDataManagerLayout',
        'lex!easyDatabaseManager',
        'css!nmodule/easyDatabaseManager/rc/css/pointDataManager'],
    function (baja,
              Widget,
              subscriberMixin,
              $,
              promise,
              DepthSubscriber,
              licenseStatusView,
              unusedPointsView,
              unwantedPointsView,
              reviewPointsView,
              deletePointsView,
              postDeleteStatusView,
              layout,
              lexicons) {

  'use strict';

      var lex = lexicons[0];
      var pointDataManagerService = null;
      var brandColorCode;
      function updateHeaderAndFooter(dom, currentView) {
            var params = currentView.getViewHeaders();
            //update headers
            $(dom).find(".PDM-step-message").text(params.message);
            $(dom).find(".PDM-feature-description-message").text(params.featureDescription);
            // update nav buttons
            $(dom).find("#PDM-nav-button-back").val(params.backBttnMsg);
            $(dom).find("#PDM-nav-button-next").val(params.nextBttnMsg);
            $(dom).find("#PDM-nav-button-cancel").val(params.cancelBttnMsg);
            $(dom).find("#PDM-nav-button-skip").val(params.skipBttnMsg);
            // unhide all buttons previously hidden by different view
            $(dom).find(".PDM-nav-button").removeClass("PDM-hide-element");
            var actionBttnStatus = currentView.getActionBttnsStatus();
            updateActionButtonsStatus(dom, actionBttnStatus);

            $(dom).find("#" + params.vid).removeClass("step-done PDM-hide-element");
            $(dom).find("#" + params.vid).addClass("step-active");
      }

  function addEventListeners() {
            var _self = this;
            var dom = _self.jq();
            $(dom).find("#PDM-nav-button-next").click({_self: _self},_self.addNextBtnListener);
            $(dom).find("#PDM-nav-button-back").click({_self: _self},_self.addBackBtnListener);
            $(dom).find("#PDM-nav-button-cancel").click({_self: _self},_self.addCancelBtnListener);
            $(dom).find("#PDM-nav-button-skip").click({_self: _self},_self.addSkipBtnListener);
  }

  function addNextBtnListener(event) {
      var _self = event.data._self;
      var currentView = _self.currentView;
      var actionBttnStatus = currentView.getActionBttnsStatus();
      var dom = _self.jq();

      // Update #PDM-nav-button-next status
      actionBttnStatus.disabledNext = $($(dom).find("#PDM-nav-button-next")).prop('disabled');


            if(currentView && (actionBttnStatus && !actionBttnStatus.disabledNext)) {
                currentView.validate()
                    .then(function(validInputs) {
                        if(validInputs) {
                            currentView.next()
                                .then(function(status) {
                                    if(status) {
                                        var viewData = currentView.getData();
                                        for (var key in viewData) {
                                            _self.pdmData[key] = viewData[key];
                                        }
                                        _self.currentView = _self.getUIView(currentView, "next");
                                        _self.updateView(currentView, "next");
                                        _self.loadView();
                                    }
                                });
                        }
                    });
            }
   }

  function addBackBtnListener(event) {
            var _self = event.data._self;
            var currentView = _self.currentView;
            var actionBttnStatus = currentView.getActionBttnsStatus();

            // Update #PDM-nav-button-next status
            // actionBttnStatus.disabledNext = $($(dom).find("#PDM-nav-button-next")).prop('disabled');

            if(currentView && (actionBttnStatus && !actionBttnStatus.disabledBack)) {
                currentView.back()
                    .then(function(status) {
                        if(status) {
                            _self.currentView = _self.getUIView(currentView, "prev");
                            _self.updateView(currentView, "prev");
                            _self.showLoadedView();
                        }
                    });
            }
  }

  function addCancelBtnListener(event) {
            var _self = event.data._self;
            var currentView = _self.currentView;
            var actionBttnStatus = currentView.getActionBttnsStatus();
            if(currentView && (actionBttnStatus && !actionBttnStatus.disabledCancel)) {
                _self.currentView.cancel();
            }
  }


        function addSkipBtnListener(event) {
            var _self = event.data._self;
            var currentView = _self.currentView;
            var actionBttnStatus = currentView.getActionBttnsStatus();
            var dom = _self.jq();

            // Update #PDM-nav-button-next status
            actionBttnStatus.disabledSkip = $($(dom).find("#PDM-nav-button-skip")).prop('disabled');


            if (currentView && (actionBttnStatus && !actionBttnStatus.disabledSkip)) {
                currentView.validate()
                    .then(function (validInputs) {
                        if (validInputs) {
                            currentView.skip()
                                .then(function (status) {
                                    if (status) {
                                        var viewData = currentView.getData();
                                        for (var key in viewData) {
                                            _self.pdmData[key] = viewData[key];
                                        }
                                        _self.currentView = _self.getUIView(currentView, "next");
                                        _self.updateView(currentView, "next");
                                        _self.loadView();
                                    }
                                });
                        }
                    });
            }
        }

  function initializeWizardSteps(dom) {
            var _self = this;
            // initializing first (license status view) screen.
            _self.wizardOrder.push(new licenseStatusView(dom, "licenseStatusView", "PDM-step1-licenseStatusView",
                //lex.get("pdm.PointDatabaseManager.licenseStatusViewMsg"),
                //lex.get("pdm.PointDatabaseManager.licenseStatusViewSummaryMsg"),
                lex.get("pdm.PointDatabaseManager.featureHeading"),
                lex.get("pdm.PointDatabaseManager.featureDescription"),
                lex.get("pdm.PointDatabaseManager.stepOneName"),
                lex.get("pdm.PointDatabaseManager.stepOneDescription"),
                lex.get("pdm.PointDatabaseManager.stepOneOfSix"),
                lex.get("pdm.PointDatabaseManager.button.back"),
                lex.get("pdm.PointDatabaseManager.button.next"),
                lex.get("pdm.PointDatabaseManager.button.cancel")));

            // initializing second (unused points view) screen.
            _self.wizardOrder.push(new unusedPointsView(dom, "unusedPointsView", "PDM-step2-unusedPointsView",
                //lex.get("pdm.PointDatabaseManager.unusedPointsViewMsg"),
                //lex.get("pdm.PointDatabaseManager.unusedPointsViewSummaryMsg"),
                lex.get("pdm.PointDatabaseManager.featureHeading"),
                lex.get("pdm.PointDatabaseManager.featureDescription"),
                lex.get("pdm.PointDatabaseManager.stepTwoName"),
                lex.get("pdm.PointDatabaseManager.stepTwoDescription"),
                lex.get("pdm.PointDatabaseManager.stepTwoOfSix"),
                lex.get("pdm.PointDatabaseManager.button.back"),
                lex.get("pdm.PointDatabaseManager.button.next"),
                lex.get("pdm.PointDatabaseManager.button.cancel"),
                lex.get("pdm.PointDatabaseManager.button.skip")));

            // initializing third (unwanted points view) screen.
            _self.wizardOrder.push(new unwantedPointsView(dom, "unwantedPointsView", "PDM-step3-unwantedPointsView",
                //lex.get("pdm.PointDatabaseManager.unwantedPointsViewMsg"),
                //lex.get("pdm.PointDatabaseManager.unwantedPointsViewSummaryMsg"),
                lex.get("pdm.PointDatabaseManager.featureHeading"),
                lex.get("pdm.PointDatabaseManager.featureDescription"),
                lex.get("pdm.PointDatabaseManager.stepThreeName"),
                lex.get("pdm.PointDatabaseManager.stepThreeDescription"),
                lex.get("pdm.PointDatabaseManager.stepThreeOfSix"),
                lex.get("pdm.PointDatabaseManager.button.back"),
                lex.get("pdm.PointDatabaseManager.button.next"),
                lex.get("pdm.PointDatabaseManager.button.cancel"),
                lex.get("pdm.PointDatabaseManager.button.skip")));

            // initializing fourth (review points view) screen.
            _self.wizardOrder.push(new reviewPointsView(dom, "reviewPointsView", "PDM-step4-reviewPointsView",
                //lex.get("pdm.PointDatabaseManager.reviewPointsViewMsg"),
                //lex.get("pdm.PointDatabaseManager.reviewPointsViewSummaryMsg"),
                lex.get("pdm.PointDatabaseManager.featureHeading"),
                lex.get("pdm.PointDatabaseManager.featureDescription"),
                lex.get("pdm.PointDatabaseManager.stepFourName"),
                lex.get("pdm.PointDatabaseManager.stepFourDescription"),
                lex.get("pdm.PointDatabaseManager.stepFourOfSix"),
                lex.get("pdm.PointDatabaseManager.button.back"),
                lex.get("pdm.PointDatabaseManager.button.next"),
                lex.get("pdm.PointDatabaseManager.button.cancel")));

            // initializing fifth (delete points view) screen.
            _self.wizardOrder.push(new deletePointsView(dom, "deletePointsView", "PDM-step5-deletePointsView",
                //lex.get("pdm.PointDatabaseManager.deletePointsViewMsg"),
                //lex.get("pdm.PointDatabaseManager.deletePointsViewSummaryMsg"),
                lex.get("pdm.PointDatabaseManager.featureHeading"),
                lex.get("pdm.PointDatabaseManager.featureDescription"),
                lex.get("pdm.PointDatabaseManager.stepFiveName"),
                lex.get("pdm.PointDatabaseManager.stepFiveDescription"),
                lex.get("pdm.PointDatabaseManager.stepFiveOfSix"),
                lex.get("pdm.PointDatabaseManager.button.back"),
                lex.get("pdm.PointDatabaseManager.button.next"),
                lex.get("pdm.PointDatabaseManager.button.cancel")));

            // initializing fifth (delete points view) screen.
            _self.wizardOrder.push(new postDeleteStatusView(dom, "postDeleteStatusView", "PDM-step6-postDeleteStatusView",
                //lex.get("pdm.PointDatabaseManager.postDeleteStatusViewMsg"),
                //lex.get("pdm.PointDatabaseManager.postDeleteStatusViewSummaryMsg"),
                lex.get("pdm.PointDatabaseManager.featureHeading"),
                lex.get("pdm.PointDatabaseManager.featureDescription"),
                lex.get("pdm.PointDatabaseManager.stepSixName"),
                lex.get("pdm.PointDatabaseManager.stepSixDescription"),
                lex.get("pdm.PointDatabaseManager.stepSixOfSix"),
                lex.get("pdm.PointDatabaseManager.button.back"),
                lex.get("pdm.PointDatabaseManager.button.startNew"),
                lex.get("pdm.PointDatabaseManager.button.cancel")));

  }

        /**
   * Description of your widget.
   *
   * @class
   * @extends module:bajaux/Widget
   * @alias module:nmodule/easyDatabaseManager/rc/EasyDatabaseManagerWidget
   */
  var EasyDatabaseManagerWidget = function () {
    var _self = this;
    Widget.apply(_self, arguments);
    subscriberMixin(_self);

    _self.wizardOrder = [];
    _self.currentView = null;
    _self.pdmData = {};
    _self.$jobSubscriber = new DepthSubscriber(5);
    _self.getUIView = function(view, type) {
        var tempview = null;
        if(view) {
            var viewIndex = _self.wizardOrder.findIndex(
                function(element) {return element.getViewHeaders().vname === view.getViewHeaders().vname});
            switch (type) {
            case 'next': {
                if(viewIndex >= _self.wizardOrder.length - 1) {
                    tempview = _self.wizardOrder[0];
                } else {
                    tempview = _self.wizardOrder[viewIndex+1];
                }
                break;
            }
            case 'prev' : {
                if(viewIndex > 0) {
                    tempview = _self.wizardOrder[viewIndex-1];
                } else {
                    tempview = _self.wizardOrder[0];
                }
                break;
            }
            default: {
                    tempview = _self.wizardOrder[0];
                    break;
                }
            }
        }
        if(tempview == null) {
            //return the first element in view
            tempview = _self.wizardOrder[0];
        }
        return tempview;
    };

    _self.loadView = function() {
        var dom = _self.jq();
        var params = _self.currentView.getViewHeaders();
        // update header and footer (nav buttons) for the view
        _self.updateHeaderAndFooter(dom, _self.currentView);
        // remove any existing view with currentView id
        $(dom).find("#" + params.vid).remove();
        // call currentView.loadView
        var viewParams = {
                    component: _self.component,
                    updateActionButtonsStatus: updateActionButtonsStatus,
                    service: pointDataManagerService,
                    subscriber: _self.$jobSubscriber,
                    notifyUser: notifyUser,
                    pdmData: _self.pdmData,
                    colorCode:brandColorCode
                };
                _self.currentView.loadView($(dom).find(".PDM-contents"), viewParams);
                $(dom).find("#" + params.vid).addClass("step-active");
    };

    _self.showLoadedView = function() {
        var dom = _self.jq();
        // update header and footer (nav buttons) for the view
        _self.updateHeaderAndFooter(dom, _self.currentView);

        var params = _self.currentView.getViewHeaders();
        $(dom).find("#" + params.vid).removeClass("step-done PDM-hide-element");
        $(dom).find("#" + params.vid).addClass("step-active");

        var viewParams = {
            pdmData: _self.pdmData
        };
        _self.currentView.showLoadedView(viewParams);
    };

            _self.updateView = function (currentView, type) {
                var params = currentView.getViewHeaders();
                var dom = _self.jq();
                //hide previous views before going next
                if (type === "next") {
                    $(dom).find("#" + params.vid).removeClass("step-active");
                    $(dom).find("#" + params.vid).addClass("step-done PDM-hide-element");
                }
                // remove existing view before going back
                if (type === "prev") {
                    $(dom).find("#" + params.vid).removeClass("step-active");
                    $(dom).find("#" + params.vid).remove();
                }
            };
            _self.addEventListeners = addEventListeners;
            _self.updateHeaderAndFooter = updateHeaderAndFooter;
            _self.addNextBtnListener = addNextBtnListener;
            _self.addBackBtnListener = addBackBtnListener;
            _self.addCancelBtnListener = addCancelBtnListener;
            _self.addSkipBtnListener = addSkipBtnListener;
            _self.initializeWizardSteps = initializeWizardSteps;
        };

        function updateActionButtonsStatus(dom, actionBttnStatus) {
            if (dom && actionBttnStatus) {
                if (actionBttnStatus.disabledBack === undefined) {
                    $(dom).find("#PDM-nav-button-back").addClass("PDM-hide-element");
                } else {
                    $(dom).find("#PDM-nav-button-back").prop("disabled", actionBttnStatus.disabledBack);
                }
                if (actionBttnStatus.disabledNext === undefined) {
                    $(dom).find("#PDM-nav-button-next").addClass("PDM-hide-element");
                } else {
                    $(dom).find("#PDM-nav-button-next").prop("disabled", actionBttnStatus.disabledNext);
                }
                if (actionBttnStatus.disabledCancel === undefined) {
                    $(dom).find("#PDM-nav-button-cancel").addClass("PDM-hide-element");
                } else {
                    $(dom).find("#PDM-nav-button-cancel").prop("disabled", actionBttnStatus.disabledCancel);
                }
                if (actionBttnStatus.disabledSkip === undefined) {
                    $(dom).find("#PDM-nav-button-skip").addClass("PDM-hide-element");
                } else {
                    $(dom).find("#PDM-nav-button-skip").prop("disabled", actionBttnStatus.disabledSkip);
                }
            }
        }

        function notifyUser(dom, message) {
            baja.outln(message);
            //hide progress.
            $(dom).find(".PDM-wizard-container").addClass("PDM-hide-element");
            $(dom).find(".PDM-error-message").html(message);
            $(dom).find(".PDM-error-notification").removeClass("PDM-hide-element");
        }

        //extend and set up prototype chain
        EasyDatabaseManagerWidget.prototype = Object.create(Widget.prototype);
        EasyDatabaseManagerWidget.prototype.constructor = EasyDatabaseManagerWidget;

        /**
         * Describe how your widget does its initial setup of the DOM.
         *
         * @param {jQuery} element the DOM element into which to load this widget
         */
        EasyDatabaseManagerWidget.prototype.doInitialize = function (dom) {
            var _self = this;
            dom.html(layout({
                        "stepMessage": lex.get("pdm.pointDataManager.welcomeMsg"),
                    "featureDescription": lex.get("pdm.pointDataManager.featureDescription"),
                    "backButtonMsg": lex.get("pdm.PointDatabaseManager.button.back"),
                    "nextButtonMsg": lex.get("pdm.PointDatabaseManager.button.next"),
                    "cancelButtonMsg": lex.get("pdm.PointDatabaseManager.button.cancel")
                }
            ));
            _self.addEventListeners();
        };

        /**
         * Describe how your widget loads in a value.
         *
         * @param value description of the value to be loaded into this widget
         */
        EasyDatabaseManagerWidget.prototype.doLoad = function (component) {
            var _self = this;
            var dom = _self.jq();
            _self.component = component;
            _self.initializeWizardSteps(dom);
            _self.currentView = _self.getUIView();
            var componentType = component.getType();

            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'getBrandColorCode'})
                .then(function (resp) {
                    brandColorCode = resp.colorCode;
                    document.getElementById("hiddenFieldIsWorkBench").value= resp.isWb;
                });

          baja.rpc({
              typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
              method: 'checkFeatureLicense'})
              .then(function (returnValue) {
                  if(!returnValue) {
                      baja.rpc({
                          typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                          method: 'checkEdgeController'
                      }).then( function(returnEdge) {
                          if (returnEdge) {
                              baja.outln("Point Database Manager runs on supervisor only, not on edge controller stations.");
                              notifyUser(dom, lex.get("pdm.PointDatabaseManager.feature.not.OnEdge"));
                          } else {
                              baja.outln("Point Database Manager is not licensed");
                              notifyUser(dom, lex.get("pdm.PointDatabaseManager.feature.not.licensed"));
                          }
                          return;
                      });
                  } else {
                      //if the controller has EDM license, show feature not available message.
                      baja.rpc({
                          typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                          method: 'checkEdgeController'
                      }).then( function(returnEdge) {
                          if (returnEdge) {
                              baja.outln("Point Database Manager runs on supervisor only, not on edge controller stations.");
                              notifyUser(dom, lex.get("pdm.PointDatabaseManager.feature.not.OnEdge"));
                          } else {
                              if (componentType.toString() === "easyDatabaseManager:EasyDatabaseManagerService") {
                                  pointDataManagerService = component;
                                  _self.loadView();
                              } else {
                                  pointDataManagerService = null;
                                  _self.getPointDataManagerService()
                                      .then(function (result) {
                                          baja.outln("ready to find unused points, retrieved EasyDatabaseManagerService: " + result);
                                          if (result instanceof Error) {
                                              notifyUser(dom, lex.get("pdm.PointDatabaseManager.no.service.message"));
                                          } else {
                                              _self.loadView();
                                          }
                                      }, function (error) {
                                          baja.outln("EasyDatabaseManagerService is unavailable: " + error);
                                          notifyUser(dom, lex.get("pdm.PointDatabaseManager.no.service.message"));
                                      })
                                      .catch(function (err) {
                                          // ORD failed to resolve
                                          baja.outln("Error finding EasyDatabaseManagerService: " + err);
                                          notifyUser(dom, lex.get("pdm.PointDatabaseManager.no.service.message"));
                                      });
                              }
                          }
                      });
                  }
              });

        };

        EasyDatabaseManagerWidget.prototype.getPointDataManagerService = function () {
            return baja.Ord.make('station:|slot:/Services').get({lease: true})
                .then(function (services) {
                    return services.getNavChildren()
                        .then(function (serviceList) {
                            var k;
                            for (k = 0; k < serviceList.length; k++) {
                                if (serviceList[k].getType().toString() === "easyDatabaseManager:EasyDatabaseManagerService") {
                                    pointDataManagerService = serviceList[k];
                                    break;
                                }
                            }
                            baja.outln('retrieved nav children: ' + serviceList.join());
                        })
                        .then(function () {
                            return new promise(function (resolve, reject) {
                                if (pointDataManagerService !== null) {
                                    resolve(pointDataManagerService);
                                } else {
                                    reject(new Error("Failed to retrieve EasyDatabaseManagerService"));
                                }
                            })
                                .catch(function (err) {
                                    baja.error('failed to retrieve available services: ' + err);
                                    return (new Error("Failed to retrieve EasyDatabaseManagerService"));
                                });
                        });
                });

        };


        /**
         * Describe what kind of data you can read out of this widget.
         *
         * @returns {promise} promise to be resolved with the current value
         */
        EasyDatabaseManagerWidget.prototype.doRead = function () {
            return promise.resolve(this.jq().find('input').val());
        };

        return EasyDatabaseManagerWidget;
    });
