/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/16/2019
 *
 */

/**
 * A module defining `deletePointsViewWidget`.
 *
 * @module nmodule/easyDatabaseManager/rc/js/datatable
 */
define(['baja!',
        'jquery',
        'underscore',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/datatable',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/datarows',
        'nmodule/easyDatabaseManager/rc/js/pagination/pagination',
        'css!nmodule/easyDatabaseManager/rc/js/pagination/pagination',
        'css!nmodule/easyDatabaseManager/rc/css/datatable'
    ],
    function (baja, $, _, template, rowsTemplate) {

        'use strict';

        var datatable = function(options) {
           /* var _self = $(this);*/

            var $parent
/*
                , _options = {
                parent:null,
                rowSelectionCallback : null
            }
*/
            ;
            var _paginationOptions = {
                pageSize: 20,
                pageRange:2
            };

            var _isdataAvailable = function(datatable_options) {
                if(datatable_options) {
                    var tableData = datatable_options.tableData;
                    if(tableData && tableData.columns && tableData.columns.length > 0 && tableData.rows) {
                        return true;
                    }
                }
              return false;
            };

            var makeDataPointsSelectable = function(event){
                var $target = $(event.target);
                var dom = event.data.dom;
                //identify if header is selected!!
                var parent = $target.parents("span.ui-column-title");
                var isHeaderColumnClicked = (parent.length > 0);
                if(isHeaderColumnClicked) {
                    if($target.hasClass("ui-chkbox-icon")) {
                        $(dom).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default")
                            .removeClass("ui-state-active");
                        $(dom).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default .ui-chkbox-icon.ui-clickable")
                            .removeClass("pi pi-check");
                    } else {
                        $(dom).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default")
                            .addClass("ui-state-active");
                        $(dom).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default .ui-chkbox-icon.ui-clickable")
                            .addClass("pi pi-check");
                    }
                } else {
                    var $header;
                    if($target.hasClass("ui-chkbox-icon")) {
                        $target.parent().toggleClass("ui-state-active");
                        $target.toggleClass("pi pi-check");

                        //if header is selected - uncheck it
                        $header = $(dom).find("span.ui-column-title div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default");
                        $header.removeClass("ui-state-active");
                        $header.find(".ui-chkbox-icon.ui-clickable").removeClass("pi pi-check");
                    } else {
                        $target.toggleClass("ui-state-active");
                        $target.find(".ui-chkbox-icon.ui-clickable").toggleClass("pi pi-check");

                        // if all rows are selected - check header as well.
                        var tbody  = $(dom).find(".ui-datatable tbody");
                        if(tbody.find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").length > 0 &&
                            (tbody.find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").length ===
                                tbody.find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.ui-state-active").length)) {
                            $header = $(dom).find("span.ui-column-title div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default");
                            $header.addClass("ui-state-active");
                            $header.find(".ui-chkbox-icon.ui-clickable").addClass("pi pi-check");
                        }
                    }
                }
                //return if row is selected or un-selected
                if(options.rowSelectionCallback) {
                    options.rowSelectionCallback(($target.hasClass("ui-chkbox-icon") ? $target.parent() : $target));
                }
            };

            var _addEventListeners = function(dom) {
                var isWorkBench = JSON.parse(document.getElementById("hiddenFieldIsWorkBench").value);
                //select or unselect individual rows
                $(dom).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").off('click', makeDataPointsSelectable);
                $(dom).find("div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").on('click',{dom: dom} , makeDataPointsSelectable);
                var stepsFour = document.getElementById("PDM-contents-step4-details");
                var allElements = stepsFour!=null?stepsFour.getElementsByClassName("cell-data-wrap"):[];
                for(var i=0;i<allElements.length;i++){
                  var itemEle=allElements[i];
                    itemEle.addEventListener("mouseover",function (event){
                        var xyRect = event.currentTarget.getBoundingClientRect();
                        var $tooltip = $(dom).find(".datatable-tooltip");
                        $tooltip.find(".datatable-tooltip-content").text($(event.target).attr("data-tooltip"));
                        var yPos = xyRect.y;
                        var xPos = xyRect.x- xyRect.width;
                        if(isWorkBench){
                            yPos = xyRect.y+35;
                            xPos = xyRect.x;
                        }
                        $tooltip.css("left", xPos + "px");
                        $tooltip.css("top",yPos + "px");
                        $tooltip.addClass("datatable-tooltip-show");
                    });
                    itemEle.addEventListener("mouseout",function (event){
                        var $tooltip = $(dom).find(".datatable-tooltip");
                        $tooltip.removeClass("datatable-tooltip-show");
                    });
                }
                if(dom.hasClass("points-table-wrapper"))
                {
                    $(".points-table-wrapper").find(".secured-selection-column").hide();
                    $(".points-table-wrapper .clear-selected-secure-point").remove();
                }else if(dom.hasClass("secure-points-table-wrapper"))
                {
                    $(".secure-points-table-wrapper").find("thead .ui-state-default .ui-selection-column").hide();
                    $(".secure-points-table-wrapper").find("tbody .ui-selection-column").hide();
                    $(".secure-points-table-wrapper").find("thead .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.checkbox-header").hide();
                    $(".secure-points-table-wrapper thead .ui-selection-column .ui-column-title").remove();
                }
            };

            var _displayPage = function(data, pagination) {
                var start = (pagination.pageNumber - 1) * pagination.pageSize;
                var end = pagination.pageNumber * pagination.pageSize;
                end = (end > pagination.totalNumber) ? pagination.totalNumber : end;
                $parent.find("tbody.ui-datatable-data tr").css('display', 'none');
                $parent.find("tbody.ui-datatable-data tr").slice(start, end).css('display', 'table-row');
            };

            var _addPagination = function() {
                $parent.pagination({
                        dataSource: options.tableData.rows,
                        callback: function(data, pagination) {
                            _displayPage(data, pagination);
                        },
                        pageSize: _paginationOptions.pageSize,
                        totalNumber: options.tableData.rows.length,
                        pageRange:_paginationOptions.pageRange
                    }
                );
            };

            var _restore_selection = function($datatable_parent) {
                if(options.updateselectedData && options.updateselectedData.length > 0) {
                    var tbody  = $datatable_parent.find(".ui-datatable tbody");
                    $.each($(tbody).find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default"),
                        function(index, elem) {
                            if(options.updateselectedData.includes($(elem).attr('data-handle'))) {
                                $(elem).toggleClass("ui-state-active");
                                $(elem).find(".ui-chkbox-icon.ui-clickable").toggleClass("pi pi-check");
                            }});
                    tbody  = $datatable_parent.find(".ui-datatable tbody");
                    if(tbody.find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").length > 0 &&
                        (tbody.find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default").length ===
                            tbody.find(".ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.ui-state-active").length)) {
                        var $header = $datatable_parent.find("span.ui-column-title div.ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default");
                        $header.addClass("ui-state-active");
                        $header.find(".ui-chkbox-icon.ui-clickable").addClass("pi pi-check");
                    }
                }
                if($datatable_parent.hasClass("secure-points-table-wrapper"))
                {
                    if(options.updateselectedData !== null)
                    {
                        $($datatable_parent).find("input[id='"+options.updateselectedData+"']").attr('checked',true);
                        $($datatable_parent).find(".clear-selected-secure-point").prop('disabled',false);
                        $($datatable_parent).find(".clear-selected-secure-point").removeClass("button-inactive");
                    }
                    if(options.rowSelectionCallback) {
                        options.rowSelectionCallback();
                    }
                }
            };

            var _replaceRows = function(rows) {
                if($parent.hasClass("secure-points-table-wrapper") && $parent.find("input[name='radioSecuredPoints']:checked").length > 0){
                    options.updateselectedData = $parent.find("input[name='radioSecuredPoints']:checked").prop("id");
                }
                $parent.find("tbody.ui-datatable-data").html(rowsTemplate({rows:rows}));
                _addPagination();
                _addEventListeners($parent);

                // restore selection
                _restore_selection($parent);

            };

            var _makeTableSortable = function(dom) {
                $(dom).find(".ui-sortable-column span").click(function(event) {
                    var $target = $(event.target);
                    if(!$target.hasClass("ui-sortable-column-icon")) {
                        $target = $target.next("span.ui-sortable-column-icon");
                    }
                    var tempData = options.tableData.rows.slice();
                    var field = $target.attr("data-name");
                    if($target.hasClass("pi-sort-up")) {
                        //sorted in ascending order. sort in descending order now
                        $target.removeClass("pi-sort-up");
                        $target.addClass("pi-sort-down");
                        tempData = _.sortBy(tempData, field).reverse();
                    } else if($target.hasClass("pi-sort-down")) {
                        //sorted in descending order. restore default order
                        $target.removeClass("pi-sort-down");
                        //do nothing just replace rows with original order
                    } else {
                        // reset sorting done on other columns - we only support single column sorting
                        $(dom).find("span.ui-sortable-column-icon").removeClass("pi-sort-up");
                        $(dom).find("span.ui-sortable-column-icon").removeClass("pi-sort-down");

                        // no sorting performed so far. Sort in ascending order
                        $target.addClass("pi-sort-up");
                        tempData = _.sortBy(tempData, field);
                    }
                     _replaceRows(tempData);
                });
            };

            var _initialise = function() {
                if(_isdataAvailable(options)) {
                    $parent = (options.parent) ? $(options.parent) :  $('<div class="table-wrapper"></div>');
                    $parent.html(template(options.tableData));
                    if(options.tableData.rows.length > 0) {
                        _addPagination();
                        _addEventListeners($parent);
                        _makeTableSortable($parent);
                    }
                    else {
                        var $tr = $('<tr><td colspan="3" class="ui-datatable-no-data"></td></tr>');
                        $tr.find(".ui-datatable-no-data").html(options.noDataMesage);
                        $parent.find("tbody.ui-datatable-data").html($tr);
                        $(".secure-points-table-wrapper").find("thead .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.checkbox-header").hide();
                        $(".points-table-wrapper .clear-selected-secure-point").remove();
                        $(".secure-points-table-wrapper thead .ui-selection-column .ui-column-title").remove();
                    }
                    // restore selection
                    _restore_selection($parent);
                }
                else {
                    throw(new Error("Unable to create table, invalid table data."));
                }
            };


            _initialise();
            return {
                create : function() {
                    _initialise();
                },
                replace : function(tableData) {
                    $parent.html(template(tableData));
                    _addEventListeners($parent);
                },
                replaceRows: function(rows) {
                    _replaceRows(rows);
                },
                destroy : function() {
                    /* Add logic to destroy (remove) datatable and clean up resources*/
                },
                addRows : function(/*start, end*/) {
                    /*Add logic to add more rows to the datatable*/
                },
                deleteRows : function(/*start, end*/) {
                    /*Add logic to delete rows from the datatable*/
                },
                showRows : function(/*start, end*/) {
                    /*Add logic to show specific rows within datatable*/
                },
                hideRows: function(/*start, end*/) {
                    /*Add logic to hide specific rows within datatable*/
                },
                updateOptions: function(opt) {
                    $.extend(options, opt);
                }
            };
        };
        return datatable;
    });
