/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/16/2019
 *
 */



define(['baja!',
        'jquery',
        'lex!easyDatabaseManager'],
    function (Baja,
              $,
              lexicons) {

        'use strict';

        var lex = lexicons[0];
        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/easyDatabaseManager/rc/js/defaultView
         */
        var view = function(dom, vname, id, message, featureDescription, summaryMessage, stepDescription, stepCount, backBttnMsg, nextBttnMsg, cancelBttnMsg, skipBttnMsg)
        {
            /*
             * Everything defined here (as part of _self) are available across all views.
            */

            var _self = this;
            //widget dom (this gives access to header and navigation buttons as well.
            _self.dom = dom;
            // All data to be passed between views will go in data{} object.
            _self.data = {};
            // Params required to initialise view
            _self.params = {
                "vname" : vname,
                "vid" : id,
                "message" : message || "",
                "featureDescription" : featureDescription || "",
                "summaryMessage" : summaryMessage || "",
                "stepDescription" : stepDescription || "",
                "stepCount" : stepCount || "",
                "backBttnMsg" : backBttnMsg || lex.get("pdm.PointDatabaseManager.button.back"),
                "nextBttnMsg" : nextBttnMsg || lex.get("pdm.PointDatabaseManager.button.next"),
                "cancelBttnMsg" : cancelBttnMsg || lex.get("pdm.PointDatabaseManager.button.cancel"),
                "skipBttnMsg" : skipBttnMsg || lex.get("pdm.PointDatabaseManager.button.skip")};
            // To track navigation button status within view
            _self.buttonstatus = {
                disabledBack : false,
                disabledNext : false,
                disabledCancel: false,
                disabledSkip: false}
        };

        //these are prototype functions and needs to be overridden in extended screens.
        /**
         * Load view of current wizard step - licenseStatusView/unusedPointsView/
         * unwantedPointsView/reviewPointsView/deletePointsView/postDeleteStatusView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        view.prototype.loadView = function(element, params) {
            throw new Error('Cannot call abstract method, please overwrite in current view: ' + element + " :: " + params);
        };

        //these are prototype functions and needs to be overridden in extended screens.
        /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        view.prototype.validate = function() {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        //these are prototype functions and needs to be overridden in extended screens.
        /**
         * clean up and save before going to next view
         *
         */
        view.prototype.next = function() {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        //these are prototype functions and needs to be overridden in extended screens.
        /**
         * clean up and save before going to previous view
         *
         */
        view.prototype.back = function() {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        //these are prototype functions and needs to be overridden in extended screens.
        /**
         * cancel operation - cancel button supported cancellable operation in current view
         *
         */
        view.prototype.cancel = function() {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        //these are prototype functions and needs to be overrdiden in extended screens.
        view.prototype.updateView = function() {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        /**
         * To pass data between views
         *
         */
        view.prototype.getData = function() {
            return this.data;
        };

        /**
         * provides navigation button status as set by view.
         *
         */
        view.prototype.getActionBttnsStatus = function () {
            return this.buttonstatus;
        };

        /**
         * Provide params needed to update view headers
         *
         */
        view.prototype.getViewHeaders = function() {
            return this.params;
        };

        return view;
    });
