/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/17/2019
 *
 */
define(['baja!',
    'jquery',
    'underscore',
    'dialogs',
    'lex!easyDatabaseManager',
    'bajaux/Widget',
    'nmodule/webEditors/rc/fe/fe',
    'nmodule/webEditors/rc/wb/table/Table',
    'nmodule/webEditors/rc/wb/table/model/columns/JsonObjectPropertyColumn',
    'nmodule/webEditors/rc/wb/table/model/TableModel',
    'nmodule/webEditors/rc/wb/table/model/Row',
    'nmodule/easyDatabaseManager/rc/js/defaultView',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/licenseStatusViewTemplate',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/LicenseSummaryTable',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/GlobalLicenseNetworkTable',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/GlobalLicenseSummaryTable',
],
    function (Baja,
        $,
        _,
        dialogs,
        lexicons,
        Widget,
        fe,
        Table,
        JsonObjectPropertyColumn,
        TableModel,
        Row,
        defaultView,
        template,
        tplPDMSummaryTable,
        tplPDMGlobalNetworkTable,
        tplPDMGlobalSummaryTable
    ) {

        'use strict';

        var lex = lexicons[0], dom, isIncreaseFooterSize = true;
        var setOfNwNotConsumingGlobalCapacity;
        var globalCapacityNetworksSet;
        var totalGlobalCapcityUsed = 0;
        var brandCode = null;
        var globalcapcitylimit = 0;
        function isUnlimited(limit) {
            if (isNaN(limit)) {
                return true;
            }
            return limit === -1;
        }


        // licenseStatusView step
        // arguments: dom, vname, id, message, summaryMessage, backBttnMsg, nextBttnMsg, cancelBttnMsg
        var licenseStatusView = function () {
            defaultView.apply(this, arguments);
            //update which buttons will be available
            this.buttonstatus = {
                disabledNext: false
            };
        };
        licenseStatusView.prototype = Object.create(defaultView.prototype, {
            'constructor': licenseStatusView
        });

        //these variables will be updated in loadView with values passed from PointDatabaseManagerWidget
        licenseStatusView.component = null;
        licenseStatusView.service = null;
        licenseStatusView.subscriber = null;
        // call this function anytime you want to update nav action button status to enable/disable them
        licenseStatusView.updateActionButtonsStatus = null;
        licenseStatusView.notifyUser = null;

        /**
         * Load view of current wizard step - licenseStatusView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        licenseStatusView.prototype.loadView = function (element, params) {
            licenseStatusView.component = params.component;
            licenseStatusView.service = params.service;
            licenseStatusView.subscriber = params.subscriber;
            licenseStatusView.updateActionButtonsStatus = params.updateActionButtonsStatus;
            licenseStatusView.notifyUser = params.notifyUser;
            dom = element;
            $(element).append(template({
                vid: this.params.vid,
                //summaryMessage: this.params.summaryMessage
                summaryMessage: this.params.summaryMessage,
                stepCount: this.params.stepCount,
                stepDescription: this.params.stepDescription,
                networkCapacityDescription: lex.get("pdm.PointDatabaseManager.licenseStatusView.available.network.points.description")
            }));
            $(element).find(".PDM-contents-step-summary").css("border-top-color", params.colorCode);
            brandCode = params.colorCode;
            $startDiscoveryJob("station:|" + params.component.getSlotPath().toString());
        };

        licenseStatusView.prototype.showLoadedView = function () {
            // If we come back to this screen do some actions if required
        };

        /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        licenseStatusView.prototype.validate = function () {
            return new Promise(function (resolve) {
                //do any validations needed here.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * clean up and save before going to next view
         *
         */
        licenseStatusView.prototype.next = function () {

            var that = this,
                targetLicenseData = [],
                isNext = true;

            return new Promise(function (resolve) {
                // do any backend communication needed here before moving to next screen.
                // resolve once all the input is validated and good to move to next screen

                //PDM-license-status-view-stage
                var globalLicenseInputBox = null;
                $.each($(dom).find("#PDM-license-status-view-stage input, .PDM-global-summary-container input"), function () {
                    /*if ( $(this).val() === "" || (isNaN($(this).val()) && $(this).val() !== "Unlimited")) {
                        isNext = false;
                    }*/
                    if (isNaN($(this).val())) {
                        isNext = false;
                    } else {
                        var item = {};
                        item["Network"] = $(this).attr('id');
                        item["Target"] = $(this).val();
                        if ($(this).val() === "") {
                            item["Target"] = "0";
                        }

                        if(this.id === "txtGlobalLicensedPoints")
                            globalLicenseInputBox = item;
                        else
                            targetLicenseData.push(item);
                    }
                });
                targetLicenseData.push(globalLicenseInputBox)
                if (isNext) {
                    that.data = {
                        licenseViewData: { targetLicense: targetLicenseData },
                        isIncreaseFooterSize : isIncreaseFooterSize
                    };
                    resolve(true);
                } else {
                    // Show Dialog

                    var dialog = dialogs.make({
                        title: lex.get("pdm.PointDatabaseManager.licenseStatusView.error.title"),
                        content: lex.get("pdm.PointDatabaseManager.licenseStatusView.error.message"),
                        ok: function (dlg) {
                            var $dialog = dlg.jq();
                            $dialog.find("div.js-dialog.ux-root").removeClass("confirm-js-dialog");
                            dlg.close();
                        }
                    });
                    dialog.show();
                    resolve(false);
                }
            });
        };

        /**
         * clean up and save before going to previous view
         *
         */
        licenseStatusView.prototype.back = function () {
            return new Promise(function (resolve) {
                // do any cleanup/backend communication needed here before moving to previous screen.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * cancel operation - cancel button supported cancellable operation in current view
         * This is invoked only if this.params.isCancelAvailable is set to true.
         */
        licenseStatusView.prototype.cancel = function () {
            // Cancel Operation
        };

        //these are prototype functions and needs to be overriden in extended screens.
        licenseStatusView.prototype.updateView = function () {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        /**
         * Kick off the discovery job using the component handles of the items selected in the previous
         * screen, and the values of the checkboxes from the options screen.
         *
         * @param {Arrary.<String>} handles - an array of handles for the components selected in the previous screen.
         * @private
         */
        function $startDiscoveryJob(nodeVal) {
            Baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'findLicenseDetails',
                args: [{ nodeOrd: nodeVal }]
            }).then(function (list) {
                Baja.rpc({
                    typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                    method: 'findGlobalCapacityDetails'
                }).then(function (globalCapacityDetails) {
                    $updateDeviceTable(list, globalCapacityDetails);
                })
            })
        };

        /**
         * Tells whether this network can consume global point capacity
         */
        function $excludedFromGlobalCapacity(networkType) {
            var driverName = networkType.substring(0, networkType.indexOf(':'));
            return setOfNwNotConsumingGlobalCapacity.has(driverName);
        }

        /**
         * Update the table with the summaries of devices currently being discovered.
         *
         * @private
         * @param {baja.Component} job - the discovery job.
         */
        function $updateDeviceTable(list, globalCapacityDetails) {
            if (!dom) {
                return;
            }

            var networkList = JSON.parse(list);
            if (!networkList || networkList.length === 0) {
                Baja.outln("Either no data is available or user do not have privilege to view data.");
                licenseStatusView.notifyUser(dom.closest(".PointDataManager-wizard"), lex.get("pdm.PointDatabaseManager.no.data.or.privilege.user.message"));
                licenseStatusView.updateActionButtonsStatus(
                    $(dom).parent(),
                    {
                        disabledNext: true
                    }
                );
                return;
            }

            var tempGlobalCapacityDetails = JSON.parse(globalCapacityDetails);
            globalcapcitylimit = tempGlobalCapacityDetails.globalPointCapacityLimit;
            var GlobalPointsSummaryElem = $(tplPDMGlobalSummaryTable({
                globalCapacityDescription: lex.get("pdm.PointDatabaseManager.licenseStatusView.available.global.points.description"),
                licensePoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.license.points"),
                unlimited: "none",
                remainingPoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.remaining.points"),
                pointsInUse: lex.get("pdm.PointDatabaseManager.licenseStatusView.points.in.use"),
                target: (tempGlobalCapacityDetails.globalPointCapacityLimit > 0) ? tempGlobalCapacityDetails.globalPointCapacityLimit : ""
            }))

            //Display Global Points summary table only if the Global points count is available and do not add border when globalCapacity not available
            var isGlobalPointCapacityAvailable = false;
            if (tempGlobalCapacityDetails.globalPointCapacityLimit)//!tempGlobalCapacityDetails.globalPointCapacityLimit) remove this comment after completion
            {
                isGlobalPointCapacityAvailable = true
                $("#PDM-step1-licenseStatusView > div.PDM-step-details").after($(GlobalPointsSummaryElem));
                $(".PDM-footer").addClass("PDM-footer-increase-margin");
            }

            //set brand color as border for the table when both the elements are created
            if (isGlobalPointCapacityAvailable) {
                $(dom).find(".PDM-global-contents-step1-details,.PDM-contents-step1-details").css("border-color", brandCode);
            }

            if (!tempGlobalCapacityDetails || tempGlobalCapacityDetails.length === 0) {
                setOfNwNotConsumingGlobalCapacity = new Set();
            } else {
                setOfNwNotConsumingGlobalCapacity = new Set(tempGlobalCapacityDetails.networksNotConsumingGlobalPointCapacity);
            }

            //get respective stage element to fill it with each network
            var networkStage = $(dom).find('#PDM-step1-licenseStatusView #PDM-license-status-view-stage');
            var networkGlobalStage = $(dom).find('#PDM-step1-licenseStatusView #PDM-global-license-status-view-stage');
            var networkCount = 0;
            globalCapacityNetworksSet = new Set();

            _.each(networkList, function (network) {
                if (network.visible) {
                    var remainingPointsClass = (network.available >= 0 || network.available == -1) ? "positive" : "negative";

                    var rowElem = $('<div class="PDM-license-status-container-row"></div>');
                    $(rowElem).attr('id', removeSpaceAndSpecialChars(network.name) + "_" + networkCount);
                    var elem = null;

                    if (!$excludedFromGlobalCapacity(network.type) && tempGlobalCapacityDetails.globalPointCapacityLimit != 0) {//!$excludedFromGlobalCapacity(network.type) remove 
                        globalCapacityNetworksSet.add(removeSpaceAndSpecialChars(network.name) + "_" + networkCount);
                        networkGlobalStage.append(rowElem);
                        elem = $(tplPDMGlobalNetworkTable({
                            driver: network.name,
                            pointsInUse: lex.get("pdm.PointDatabaseManager.licenseStatusView.points.in.use"),
                            pointsInUseValue: network.existing,
                            pointsInUseValueFormatted: network.existing.toLocaleString(),
                        }));
                    } else {
                        networkStage.append(rowElem);
                        elem = $(tplPDMSummaryTable({
                            driver: network.name,
                            target: network.limit === -1 ? "" : network.limit,
                            unlimited: "none",
                            licensePoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.license.points"),
                            globalPointsCapacity: lex.get("pdm.PointDatabaseManager.licenseStatusView.global.points.capacity"),
                            pointsInUse: lex.get("pdm.PointDatabaseManager.licenseStatusView.points.in.use"),
                            remainingPoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.remaining.points"),
                            pointsInUseValue: network.existing,
                            pointsInUseValueFormatted: network.existing.toLocaleString(),
                            remainingPointsValue: network.available === -1 ? 0 : network.available.toLocaleString(),
                            remainingPointsClass: remainingPointsClass
                        }));
                        var textbox = elem.find('.PDM-license-status-text');
                        $(textbox).attr('id', "txt" + removeSpaceAndSpecialChars(network.name) + "_" + networkCount);
                        networkCount++;
                        $addTextBoxEvent(textbox, network, false);
                    }
                    rowElem.append(elem);
                }
            });

            /*$(".PDM-license-status-text").attr("placeholder", availableGlobalCapacity);*/
            updatePlaceholder(tempGlobalCapacityDetails);
            $addTextBoxEvent($(dom).find("#txtGlobalLicensedPoints"), null, true);

            isIncreaseFooterSize = true;
            //do not display block if there are no networks in drivers with individual licensing and drivers with global licensing respectively.
            if (!($(dom).find('#PDM-step1-licenseStatusView #PDM-license-status-view-stage .PDM-license-status-container-row').length)) {
                $(dom).find(".PDM-contents-step1-details").css("display", "none");
                $(dom).find(".PDM-global-contents-step1-details").addClass("PDM-hide-element-set-margin");
                $(".PDM-footer").removeClass("PDM-footer-increase-margin");
                isIncreaseFooterSize = false;
            }
            if (!($(dom).find('#PDM-step1-licenseStatusView #PDM-global-license-status-view-stage .PDM-license-status-container-row').length)) {
                $(dom).find(".PDM-global-contents-step1-details").css("display", "none");
                $(dom).find(".PDM-contents-step1-details").addClass("PDM-hide-element-set-margin");
                $(".PDM-footer").removeClass("PDM-footer-increase-margin");
                isIncreaseFooterSize = false;
            }
        };

        function updatePlaceholder(tempGlobalCapacityDetails) {
            //set GLobal points summary including licensedPoints, used and remaining count
            //get all network details under Drivers which are using global capacity then calculate remaining Points in use
            var globalCapacityPointsInUse = 0;
            totalGlobalCapcityUsed = 0;
            var tmpGlobalRemainingPointsClass = "";
            Baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'findLicenseDetails',
                args: [{ nodeOrd: "station:|slot:/Drivers" }]
            }).then(function (list) {
                var networkList = JSON.parse(list);
                _.each(networkList, function (network) {
                    if (!$excludedFromGlobalCapacity(network.type)) {
                        globalCapacityPointsInUse = network.existing == -1 ? globalCapacityPointsInUse : parseInt(network.existing, 10) + globalCapacityPointsInUse;
                    }
                });
                $(".PDM-global-summary-container .points-in-use").html(globalCapacityPointsInUse);
                totalGlobalCapcityUsed = globalCapacityPointsInUse;
                var tmpTotalGlobalCapacityRemaining = parseInt(tempGlobalCapacityDetails.globalPointCapacityLimit, 10) - parseInt(globalCapacityPointsInUse, 10);
                $(dom).find(".PDM-global-summary-container .remaining-points").text(tmpTotalGlobalCapacityRemaining);
                if (tmpTotalGlobalCapacityRemaining >= 0)
                    tmpGlobalRemainingPointsClass = "positive";
                else
                    tmpGlobalRemainingPointsClass = "negative"
                $(dom).find(".PDM-global-summary-container .remaining-points").addClass(tmpGlobalRemainingPointsClass);
            });
        };

        function removeSpaceAndSpecialChars(networkName) {
            return networkName.trim().replace(/[^A-Z0-9]+/ig, "_");
        };

        function $addTextBoxEvent(textbox, network, isGlobalPointsTextbox) {
            $(textbox).bind("change keyup", function () {
                var val = $(this).val();

                if ((!isNaN(val) && parseInt(val, 10))
                    || val.toLowerCase() === "unlimited"
                    || val.trim() === ""
                    || parseInt(val, 10) === 0) {
                    $updatePointsCount($(textbox), network, isGlobalPointsTextbox);
                    if (parseInt(val, 10) <= 0) {
                        licenseStatusView.updateActionButtonsStatus(
                            $(dom).parent(),
                            {
                                disabledNext: true,
                            }
                        );
                    } else {
                        licenseStatusView.updateActionButtonsStatus(
                            $(dom).parent(),
                            {
                                disabledNext: false,
                            }
                        );
                    }
                } else {
                    var id = $(textbox).get(0).id.replace("txt", "");
                    $(dom).find("#" + id + " .remaining-points").removeClass("positive negative").addClass("");
                    $(dom).find("#" + id + " .remaining-points").text(lex.get('pdm.PointDatabaseManager.licenseStatusView.Unlimited'));

                    licenseStatusView.updateActionButtonsStatus(
                        $(dom).parent(),
                        {
                            disabledNext: true,
                        }
                    );
                }
            });
        }

        function $updatePointsCount(textbox, network, isGlobalPointsTextbox) {
            var id = $(textbox).get(0).id.replace("txt", ""),
                existing = isGlobalPointsTextbox ? totalGlobalCapcityUsed : network.existing,
                newVal = parseInt($(textbox).val(), 10),
                unlimited = isUnlimited(newVal),
                remaining,
                remainingPointsClass = "";

            //In license points text box, if user enters non-numeric value or it is empty,
            //then unlimited will be true. For calculation purpose we are making newVal as 0.
            // Other newVal will have special characters and it will lead to error or wrong calculation.
            if (unlimited) {
                newVal = 0;
            }
            //If user enters zero or negative value
            if (newVal < 0) {
                unlimited = true;
            }

            if (unlimited) {
                remaining = 0;
            }
            else {
                remaining = newVal - existing;
            }
            remainingPointsClass = getRemainingPointsClass(remaining, network, isGlobalPointsTextbox, newVal);
            remaining = remaining.toLocaleString();
            if (!isGlobalPointsTextbox) {
                $(dom).find("#" + id + " .remaining-points").removeClass("positive negative").addClass(remainingPointsClass);
                $(dom).find("#" + id + " .remaining-points").text(remaining);
            }
            else {
                $(dom).find(".PDM-global-summary-container .remaining-points").removeClass("positive negative").addClass(remainingPointsClass);
                $(dom).find(".PDM-global-summary-container .remaining-points").text(remaining);
            }
        };

        function getRemainingPointsClass(remaining, network, isGlobalPointsTextbox, newVal)
        {
            if (remaining >= 0) {
                if((isGlobalPointsTextbox && (parseInt(newVal,10) - parseInt(globalcapcitylimit)) <= 0) ||
                (!isGlobalPointsTextbox && (network.limit == -1 || (parseInt(newVal,10) - parseInt(network.limit)) <= 0)))
                    return "positive";
                else
                    return "negative"
            } else {
                return "negative";
            }
        }

        return licenseStatusView;
    });
