/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/19/2019
 *
 */

define(['baja!',
    'jquery',
    'underscore',
    'lex!easyDatabaseManager',
    'bajaux/Widget',
    'nmodule/webEditors/rc/fe/fe',
    'nmodule/webEditors/rc/wb/table/Table',
    'nmodule/webEditors/rc/wb/table/model/columns/JsonObjectPropertyColumn',
    'nmodule/webEditors/rc/wb/table/model/TableModel',
    'nmodule/webEditors/rc/wb/table/model/Row',
    'nmodule/easyDatabaseManager/rc/js/defaultView',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/postDeleteStatusViewTemplate',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/postDeleteLicenseStatusTable',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/GlobalPostDeleteNetworkTable',
    'hbs!nmodule/easyDatabaseManager/rc/hbs/GlobalPostDeleteSummaryTable'
],
    function (baja,
        $,
        _,
        lexicons,
        Widget,
        fe,
        Table,
        JsonObjectPropertyColumn,
        TableModel,
        Row,
        defaultView,
        template,
        tplPDMSummaryTable,
        tplPDMGlobalNetworkTable,
        tplPDMGlobalSummaryTable) {
        'use strict';

        var lex = lexicons[0];
        var globalCapacityNetworksSet;
        var setOfNwNotConsumingGlobalCapacity;
        var availableGlobalCapacity = 0;
        var globalcapcitylimit = 0;
        var totalGlobalCapcityUsed = 0;
        var brandCode = null;
        var dom;
        function isUnlimited(limit) {
            if (isNaN(limit)) {
                return true;
            }
            return limit === -1;
        }


        var targetLicenseData = null;


        // postDeleteStatusView step
        // arguments: dom, vname, id, message, summaryMessage, backBttnMsg, nextBttnMsg, cancelBttnMsg
        var postDeleteStatusView = function () {
            defaultView.apply(this, arguments);

            //update which buttons will be available
            this.buttonstatus = {
                disabledNext: false
            };
        };
        postDeleteStatusView.prototype = Object.create(defaultView.prototype,
            { 'constructor': postDeleteStatusView }
        );

        //these variables will be updated in loadView with values passed from PointDatabaseManagerWidget
        postDeleteStatusView.component = null;
        postDeleteStatusView.service = null;
        postDeleteStatusView.subscriber = null;
        // call this function anytime you want to update nav action button status to enable/disable them
        postDeleteStatusView.updateActionButtonsStatus = null;

        /**
         * Load view of current wizard step - postDeleteStatusView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        postDeleteStatusView.prototype.loadView = function (element, params) {
            postDeleteStatusView.component = params.component;
            postDeleteStatusView.service = params.service;
            postDeleteStatusView.subscriber = params.subscriber;
            postDeleteStatusView.updateActionButtonsStatus = params.updateActionButtonsStatus;
            targetLicenseData = params.pdmData.licenseViewData.targetLicense;


            $(element).append(template({
                vid: this.params.vid,
                summaryMessage: this.params.summaryMessage,
                stepCount: this.params.stepCount,
                stepDescription: this.params.stepDescription,
                networkCapacityDescription: lex.get("pdm.PointDatabaseManager.licenseStatusView.available.network.points.description")
            }));
            $(element).find(".PDM-contents-step-summary").css("border-top-color", params.colorCode);
            var that = this;
            dom = element;
            brandCode = params.colorCode;
            that.$startDiscoveryJob("station:|" + params.component.getSlotPath().toString());
        };

        postDeleteStatusView.prototype.showLoadedView = function () {
            // If We come back to this view
        };
        /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        postDeleteStatusView.prototype.validate = function () {
            baja.outln("inside: postDeleteStatusView.prototype.validate ...");
            return new Promise(function (resolve) {
                //do any validations needed here.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * clean up and save before going to next view
         *
         */
        postDeleteStatusView.prototype.next = function () {
            baja.outln("inside: postDeleteStatusView.prototype.next ...");
            return new Promise(function (resolve) {
                // do any backend communication needed here before moving to next screen.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * clean up and save before going to previous view
         *
         */
        postDeleteStatusView.prototype.back = function () {
            baja.outln("inside: postDeleteStatusView.prototype.back ...");
            return new Promise(function (resolve) {
                // do any cleanup/backend communication needed here before moving to previous screen.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * cancel operation - cancel button supported cancellable operation in current view
         * This is invoked only if this.params.isCancelAvailable is set to true.
         */
        postDeleteStatusView.prototype.cancel = function () {
            /*
             * if cancel button is available, the action should be implemented here.
            */
        };

        //these are prototype functions and needs to be overriden in extended screens.
        postDeleteStatusView.prototype.updateView = function () {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        /**
         * Kick off the discovery job using the component handles of the items selected in the previous
         * screen, and the values of the checkboxes from the options screen.
         *
         * @param {Arrary.<String>} handles - an array of handles for the components selected in the previous screen.
         * @private
         */
        postDeleteStatusView.prototype.$startDiscoveryJob = function (nodeVal) {
            var that = this;
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'findLicenseDetails',
                args: [{ nodeOrd: nodeVal }]
            })
                .then(function (list) {
                    baja.rpc({
                        typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                        method: 'findGlobalCapacityDetails'
                    }).then(function (globalCapacityDetails) {
                        that.$updateDeviceTable(list, globalCapacityDetails);
                    })
                })
        };


        /**
         * Update the table with the summaries of devices currently being discovered.
         *
         * @private
         * @param {baja.Component} job - the discovery job.
         */
        function removeSpaceAndSpecialChars(networkName) {
            return networkName.trim().replace(/[^A-Z0-9]+/ig, "_");
        }
        postDeleteStatusView.prototype.$updateDeviceTable = function (list, globalCapacityDetails) {
            var networkList = JSON.parse(list);

            var that = this,
                dom = that.dom,
                networkStage = dom.find('#PDM-license-status-after-delete-stage');

            if (!dom) {
                return;
            }
            var tempGlobalCapacityDetails = JSON.parse(globalCapacityDetails);
            globalcapcitylimit = tempGlobalCapacityDetails.globalPointCapacityLimit;
            var GlobalPointsSummaryElem = $(tplPDMGlobalSummaryTable({
                globalCapacityDescription: lex.get("pdm.PointDatabaseManager.licenseStatusView.available.global.points.description"),
                licensePoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.license.points"),
                unlimited: "none",
                remainingPoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.remaining.points"),
                pointsInUse: lex.get("pdm.PointDatabaseManager.licenseStatusView.points.in.use"),
                target: (tempGlobalCapacityDetails.globalPointCapacityLimit > 0) ? tempGlobalCapacityDetails.globalPointCapacityLimit : "0"
            }))

            //Display Global Points summary table only if the Global points count is available and do not add border when globalCapacity not available
            var isGlobalPointCapacityAvailable = false;
            if (tempGlobalCapacityDetails.globalPointCapacityLimit) {
                isGlobalPointCapacityAvailable = true;
                $("#PDM-step6-postDeleteStatusView > div.PDM-step-details").after($(GlobalPointsSummaryElem));
                $(".PDM-footer").addClass("PDM-footer-increase-margin");
            }

            //set brand color as border for the table when both the elements are created
            if (isGlobalPointCapacityAvailable) {
                $(dom).find(".PDM-global-contents-step1-details,.PDM-contents-step4-details").css("border-color", brandCode);
            }

            if (!tempGlobalCapacityDetails || tempGlobalCapacityDetails.length === 0) {
                setOfNwNotConsumingGlobalCapacity = new Set();
            } else {
                var networksNotConsumingGlobalPointCapacity = tempGlobalCapacityDetails.networksNotConsumingGlobalPointCapacity;
                setOfNwNotConsumingGlobalCapacity = new Set(networksNotConsumingGlobalPointCapacity);
            }

            //get respective stage element to fill it with each network
            var networkStage = $(dom).find('#PDM-step6-postDeleteStatusView #PDM-license-status-after-delete-stage');
            var networkGlobalStage = $(dom).find('#PDM-step6-postDeleteStatusView #PDM-global-license-status-view-stage');
            globalCapacityNetworksSet = new Set();
            var networkCount = 0;

            _.each(networkList, function (network) {
                var val;

                $.each(targetLicenseData, function (key, value) {
                    if(value == null)
                        return;
                    if (value.Network === "txt" + removeSpaceAndSpecialChars(network.name) + "_" + networkCount) {
                        val = value.Target;
                    }else if(value.Network === "txtGlobalLicensedPoints"){
                        availableGlobalCapacity = value.Target;
                    }  
                    return;
                });

                if (network.visible) {
                    var remainingPointsClass = "";
                    var elem = null;
                    var rowElem = $('<div class="PDM-license-status-container-row"></div>');

                    var remainingPoints = parseInt(val, 10) - network.existing;
                    var remainingPointsClass = (val == 0 || (remainingPoints >= -1 && network.limit == -1) || 
                    (network.limit != -1 && (parseInt(val, 10) - parseInt(network.limit) <= 0))) ? "positive" : "negative";
                    $(rowElem).attr('id', "Summary" + removeSpaceAndSpecialChars(network.name) + "_" + networkCount);


                    if (!$excludedFromGlobalCapacity(network.type) && tempGlobalCapacityDetails.globalPointCapacityLimit != 0) {
                        globalCapacityNetworksSet.add("Summary" + removeSpaceAndSpecialChars(network.name) + "_" + networkCount);
                        networkGlobalStage.append(rowElem);
                        elem = $(tplPDMGlobalNetworkTable({
                            driver: network.name,
                            pointsInUse: lex.get("pdm.PointDatabaseManager.licenseStatusView.points.in.use"),
                            pointsInUseValue: network.existing,
                            pointsInUseValueFormatted: network.existing.toLocaleString(),
                        }));
                    }
                    else {
                        networkStage.append(rowElem);
                        var elem = $(tplPDMSummaryTable({
                            driver: network.name,
                            target: val == 0 ? "none" : val,
                            licensePoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.license.points"),
                            globalPointsCapacity: lex.get("pdm.PointDatabaseManager.licenseStatusView.global.points.capacity"),
                            pointsInUse: lex.get("pdm.PointDatabaseManager.licenseStatusView.points.in.use"),
                            remainingPoints: lex.get("pdm.PointDatabaseManager.licenseStatusView.remaining.points"),
                            pointsInUseValue: network.existing,
                            pointsInUseValueFormatted: network.existing.toLocaleString(),
                            remainingPointsValue: val ==0 ? 0 : remainingPoints.toLocaleString(),
                            remainingPointsClass: remainingPointsClass
                        }));
                    networkCount++;
                    }
                    rowElem.append(elem);
                }
            });
            updatePlaceholder(tempGlobalCapacityDetails);

            //do not display block if there are no networks in drivers with individual licensing and drivers with global licensing respectively.
            if (!($(dom).find('#PDM-step6-postDeleteStatusView #PDM-license-status-after-delete-stage .PDM-license-status-container-row').length)) {
                $(dom).find(".PDM-contents-step4-details").css("display", "none");
                $(dom).find(".PDM-global-contents-step1-details").addClass("PDM-hide-element-set-margin");
            }
            if (!($(dom).find('#PDM-step6-postDeleteStatusView #PDM-global-license-status-view-stage .PDM-license-status-container-row').length)) {
                $(dom).find(".PDM-global-contents-step1-details").css("display", "none");
                $(dom).find(".PDM-contents-step4-details").addClass("PDM-hide-element-set-margin");
            }

        };

        function updatePlaceholder(tempGlobalCapacityDetails) {
            //set GLobal points summary including licensedPoints, used and remaining count
            //get all network details under Drivers which are using global capacity then calculate remaining Points in use
            var globalCapacityPointsInUse = 0;
            totalGlobalCapcityUsed = 0;
            var tmpGlobalRemainingPointsClass = "";
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'findLicenseDetails',
                args: [{ nodeOrd: "station:|slot:/Drivers" }]
            }).then(function (list) {
                var networkList = JSON.parse(list);
                _.each(networkList, function (network) {
                    if (!$excludedFromGlobalCapacity(network.type)) {
                        globalCapacityPointsInUse = network.existing == -1 ? globalCapacityPointsInUse : parseInt(network.existing, 10) + globalCapacityPointsInUse;
                    }
                });
                //set user entered value for the global capacity limit
                $("#PDM-step6-postDeleteStatusView .Global-Licensed-Points").html(availableGlobalCapacity);
                $(".PDM-global-summary-container .points-in-use").html(globalCapacityPointsInUse);
                totalGlobalCapcityUsed = globalCapacityPointsInUse;
                var tmpTotalGlobalCapacityRemaining = (parseInt(availableGlobalCapacity, 10) - parseInt(globalCapacityPointsInUse, 10));
                $(dom).find(".PDM-global-summary-container .remaining-points").text(tmpTotalGlobalCapacityRemaining);
                if (tmpTotalGlobalCapacityRemaining >= 0 && (parseInt(availableGlobalCapacity, 10) - parseInt(tempGlobalCapacityDetails.globalPointCapacityLimit, 10)) <= 0)
                    tmpGlobalRemainingPointsClass = "positive";
                else
                    tmpGlobalRemainingPointsClass = "negative"
                $(dom).find(".PDM-global-summary-container .remaining-points").addClass(tmpGlobalRemainingPointsClass);
            });
        };


        /**
         * Tells whether this network can consume global point capacity
         */
        function $excludedFromGlobalCapacity(networkType) {
            var driverName = networkType.substring(0, networkType.indexOf(':'));
            return setOfNwNotConsumingGlobalCapacity.has(driverName);
        }
        return postDeleteStatusView;
    });
