/*
 *
 *  * Copyright (c) 2019. Honeywell International, Inc. All Rights Reserved.
 *    created by H148225 on 7/19/2019
 *
 */

define(['baja!',
        'bajaux/Widget',
        'bajaux/mixin/subscriberMixIn',
        'jquery',
        'underscore',
        'Promise',
        'dialogs',
        'nmodule/js/rc/csrf/csrfUtil',
        'nmodule/easyDatabaseManager/rc/js/datatable',
        'nmodule/easyDatabaseManager/rc/js/defaultView',
        'hbs!nmodule/easyDatabaseManager/rc/hbs/reviewPointsViewTemplate',
        'lex!easyDatabaseManager',
        'css!nmodule/easyDatabaseManager/rc/css/styles'],
    function (baja,
              Widget,
              subscriberMixIn,
              $,
              _,
              promise,
              dialogs,
              csrfUtil,
              datatable,
              defaultView,
              template,
              lexicons) {

        'use strict';

        //lexicons
        var lex = lexicons[0];

        var columns = [], pointsSize = 0,securePointsSize = 0, dom, pointsData,securePointsData, labelSearchCriteriaList = [],
            selectedPointsData = new Array(),selectedSecurePoint = null,
            pointsDataTable,securePointsDataTable,
            userTypingTimeout,
            MIN_PROGRESS = 0,
            MAX_PROGRESS = 100,
            selectedCheckBoxCount=0,
            PROGRESS_BAR_SUCCESS_CSS_CLASS = 'wizard-progress-bar-success',
            PROGRESS_BAR_FAIL_CSS_CLASS = 'wizard-progress-bar-fail',
            PROGRESS_BAR_RUNNING_CSS_CLASS = 'wizard-progress-bar-running';

        // reviewPointsView step
        // arguments: dom, vname, id, message, summaryMessage, backBttnMsg, nextBttnMsg, cancelBttnMsg
        var reviewPointsView = function () {
            defaultView.apply(this, arguments);

            //update which buttons will be available
            this.buttonstatus = {
                disabledBack : false,
                disabledNext : false };
        };
        reviewPointsView.prototype = Object.create(defaultView.prototype,
            { 'constructor': reviewPointsView }
            );

        //these variables will be updated in loadView with values passed from PointDatabaseManagerWidget
        reviewPointsView.component = null;
        reviewPointsView.service = null;
        reviewPointsView.subscriber = null;
        // call this function anytime you want to update nav action button status to enable/disable them
        reviewPointsView.updateActionButtonsStatus = null;

        /**
         * Load view of current wizard step - reviewPointsView
         * @param element: dom element the view will be appended to.
         * @param params: javascript object with widget parameters required to load the view within the widget
         */
        reviewPointsView.prototype.loadView = function(element, params) {
            pointsSize = 0;
            selectedCheckBoxCount=0;
            reviewPointsView.component = params.component;
            reviewPointsView.service = params.service;
            reviewPointsView.subscriber = params.subscriber;
            reviewPointsView.updateActionButtonsStatus = params.updateActionButtonsStatus;
            labelSearchCriteriaList = params.pdmData.unwantedPointsViewData.labelSearchCriteriaList;

            selectedPointsData = new Array();
            selectedSecurePoint = null;
            columns = [{
                name:lex.get("pdm.PointDatabaseManager.deletePointsView.column.name.label"),
                field:"name"
            }, {
                name:lex.get("pdm.PointDatabaseManager.deletePointsView.column.path.label"),
                field:"path"
            }
            ];

            dom = element;
            $(element).append(template({
                vid: this.params.vid,
                summaryMessage: this.params.summaryMessage,
                stepCount: this.params.stepCount,
                stepDescription: this.params.stepDescription,
                searchText:lex.get("pdm.PointDatabaseManager.reviewPointsView.searchMsg"),
                pointsSize: pointsSize,
                totalCountLabel: lex.get("pdm.PointDatabaseManager.deletePointsView.display.delete.points"),
                    selectedCheckboxCountText1: lex.get("pdm.PointDatabaseManager.stepFour.selectedCheckBoxCount.text1"),
                    selectedCheckboxCountText2: lex.get("pdm.PointDatabaseManager.stepFour.selectedCheckBoxCount.text2"),
                    pointsWithoutEsignature: lex.get("pdm.PointDatabaseManager.stepFour.pointsWithoutEsignature"),
                    pointsWithEsignature: lex.get("pdm.PointDatabaseManager.stepFour.pointsWithEsignature"),
                selectedCheckboxCount: selectedCheckBoxCount}
                ));
            $(element).find(".PDM-contents-step-summary").css("border-top-color",params.colorCode);
            this.attachButtonEvents();
            reviewPointsView.prototype.attachButtonEvents();
            displayPoints();
            changeNextButtonState();
        };

        reviewPointsView.prototype.attachButtonEvents = function(){
            $("#button-tab-unsecured-points").ready(function(event){
                $(".secure-points-table-wrapper").hide();
                $("#button-tab-unsecured-points").addClass("active");
                $("#button-tab-secured-points").removeClass("active");
                $(".points-table-wrapper .clear-selected-secure-point").remove();
                $(".points-table-wrapper").show();
                $(".points-meta-data").show();
                $("#pdm-secure-search-input").hide();
                updatePointsMetaData();
                event.preventDefault();
            });
            $("#button-tab-secured-points").on('click',
                function(event){
                    //hide the secured points tab and add active class to the button
                    $(".points-table-wrapper").hide();
                    $("#button-tab-secured-points").addClass("active");
                    $("#button-tab-unsecured-points").removeClass("active");
                    $(".secure-points-table-wrapper").show();
                    $(".secure-points-table-wrapper").find("tbody .ui-selection-column").hide();
                    $(".secure-points-table-wrapper").find("thead .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.checkbox-header").hide();
                    $(".secure-points-table-wrapper thead .ui-selection-column .ui-column-title").remove();
                    $("#pdm-search-input").hide();
                    $("#pdm-secure-search-input").show();
                    updatePointsMetaData();
                    event.preventDefault();
                });
            $("#button-tab-unsecured-points").on('click',
                function(event){
                    $(".secure-points-table-wrapper").hide();
                    $("#button-tab-unsecured-points").addClass("active");
                    $("#button-tab-secured-points").removeClass("active");
                    $(".points-table-wrapper").show();
                    $(".points-meta-data").show();
                    $(".points-table-wrapper").find(".secured-selection-column").hide();
                    $("#pdm-search-input").show();
                    $("#pdm-secure-search-input").hide();
                    updatePointsMetaData();
                    event.preventDefault();
                });
            attachRadioButtonEvent();

    };
        function attachRadioButtonEvent(){
            $(dom).find(".clear-selected-secure-point").on('click',
                function(event) {
                    $(".secure-points-table-wrapper").find("input[id='"+selectedSecurePoint+"']").prop('checked',false);
                    $(dom).find(".clear-selected-secure-point").prop('disabled',true);
                    $(dom).find(".clear-selected-secure-point").addClass("button-inactive");
                    selectedSecurePoint = null;
                    changeNextButtonState();
                });
            $(".secure-points-table-wrapper input[name='radioSecuredPoints']").click(function() {
                if ($(this).is(':checked')) {
                    selectedSecurePoint = this.getAttribute("id");
                    $(dom).find(".clear-selected-secure-point").prop('disabled',false);
                    $(dom).find(".clear-selected-secure-point").removeClass("button-inactive");
                    changeNextButtonState();
                }});
        };

        reviewPointsView.prototype.showLoadedView = function () {
            // If we come back
        };
        /**
         * validate user input before proceeding to next screen.
         * This is called before view.prototype.next is called.
         *
         * @returns {boolean}: return true/false based on validity of user input
         */
        reviewPointsView.prototype.validate = function() {
            baja.outln("inside: reviewPointsView.prototype.validate ...");
            return new promise(function(resolve) {
                //do any validations needed here.
                //resolve once all the input is validated and good to move to next screen
                resolve(true);
            });
        };

        /**
         * clean up and save before going to next view
         *
         */
        reviewPointsView.prototype.next = function() {
            baja.outln("inside: reviewPointsView.prototype.next ...");

            reviewPointsView.updateActionButtonsStatus(
                $(dom).parent(),
                {
                    disabledBack: false,
                    disabledNext: true
                }
            );

            return new promise(function(resolve) {
                // do any backend communication needed here before moving to next screen.
                //resolve once all the input is validated and good to move to next screen

                if(changeNextButtonState()) {
                    updateSelectedPoints(resolve);
                }
            });
        };

        /**
         * clean up and save before going to previous view
         *
         */
        reviewPointsView.prototype.back = function() {
            var that = this;
            baja.outln("inside: reviewPointsView.prototype.back ...");
            return new promise(function(resolve) {
                // do any cleanup/backend communication needed here before moving to previous screen.
                //resolve once all the input is validated and good to move to next screen.
                that.data = {
                    unwantedPointsViewData: {
                        labelSearchCriteriaList: []
                    }
                };
                resolve(true);
            });
        };

        /**
         * cancel operation - cancel button supported cancellable operation in current view
         * This is invoked only if this.params.isCancelAvailable is set to true.
         */
        reviewPointsView.prototype.cancel = function() {
            /*
             * if cancel button is available, the action should be implemented here.
            */
        };

        //these are prototype functions and needs to be overriden in extended screens.
        reviewPointsView.prototype.updateView = function() {
            throw new Error('Cannot call abstract method, please overwrite in current view');
        };

        function displayPoints() {
            var criteria = [],
                isTextboxesEmpty = false;

            // read xml file
            baja.rpc({
                typeSpec: 'easyDatabaseManager:PointDataManagerRpc',
                method: 'findAllItems',
                args: [{
                    nodeOrd: "station:|" + reviewPointsView.component.getSlotPath().toString(),
                    labels: labelSearchCriteriaList
                }]
            }).then(function (ord) {
                var job = baja.Ord.make("station:|" + ord).get({lease: true});
                return job;
            }).then(completeJob)
                .then(function () {
                        updateData();
                        makeDataSearchable();
                    },
                    function () {
                        //notify user.
                        //notifyUser('failed station backup.')
                    }
                );

        }


        function completeJob(job) {
            return new Promise(function (resolve, reject) {
                function checkIfJobIsComplete() {
                    switch (job.get('jobState').getTag()) {
                        case 'success':
                            updateProgressBar(MAX_PROGRESS, true);
                            $(dom).parent(),
                                {
                                    disabledBack: false,
                                    disabledNext: false,
                                }
                            return resolve();
                        case 'failed' :
                            updateProgressBar(MAX_PROGRESS, false);
                            reviewPointsView.updateActionButtonsStatus(
                                $(dom).parent(),
                                {
                                    disabledBack: false,
                                    disabledNext: true,
                                }
                            );
                            return reject();
                        case 'cancelled':
                            updateProgressBar(MAX_PROGRESS, false);
                            reviewPointsView.updateActionButtonsStatus(
                                $(dom).parent(),
                                {
                                    disabledBack: false,
                                    disabledNext: true,
                                }
                            );
                            return reject();
                        case 'running':
                            updateProgressBar(job.get('progress'));
                            break;
                        default :
                            break;
                    }
                }
                var sub = new baja.Subscriber();
                sub.attach('changed', checkIfJobIsComplete);
                sub.subscribe(job).then(checkIfJobIsComplete);
            });
        }

        function getSelectedData(elem) {
            if($(elem).hasClass("checkbox-header")) {
                //header selected/unselected
                var tmpCount = 0;
                if($(elem).hasClass("ui-state-active")) {
                    $.each($(dom).find(".points-table-wrapper .ui-datatable tbody .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default.ui-state-active"),
                        function(index, element) {
                            if($(element).attr('data-handle') !== undefined){
                                tmpCount++;
                                selectedPointsData.push($(element).attr('data-handle'));}}
                    );
                    selectedCheckBoxCount = tmpCount === pointsSize ? tmpCount : tmpCount + selectedCheckBoxCount;

                } else {
                    var addRows = new Array(), removeRows = new Array();
                    $.each($(dom).find(".points-table-wrapper .ui-datatable tbody .ui-chkbox-box.ui-widget.ui-corner-all.ui-state-default"),
                        function(index, element) {
                            if($(element).attr('data-handle') !== undefined) {
                                tmpCount++;
                                if ($(element).hasClass("ui-state-active")) {
                                    addRows.push($(element).attr('data-handle'));
                                } else if (!$(element).hasClass("ui-state-active")) {
                                    removeRows.push($(element).attr('data-handle'));
                                }
                            }
                        }
                    );
                    if( tmpCount === pointsSize ) {
                        selectedCheckBoxCount = 0;
                    }else {
                        if (selectedCheckBoxCount > tmpCount) {
                            selectedCheckBoxCount = selectedCheckBoxCount - tmpCount;
                        } else {
                            selectedCheckBoxCount = tmpCount - selectedCheckBoxCount;
                        }
                    }
                    selectedPointsData =  _.union(selectedPointsData, addRows);
                    selectedPointsData = _.difference(selectedPointsData, removeRows);
                }
            } else {
                if($(elem).hasClass("ui-state-active")) {
                    selectedCheckBoxCount++;
                    selectedPointsData.push($(elem).attr('data-handle'));
                } else {
                    selectedCheckBoxCount--;
                    var handle = $(elem).attr('data-handle');
                    selectedPointsData = selectedPointsData.filter(function (value) {
                        return value !== handle;
                    });
                }
            }
            $(dom).find('#selected-checkbox-count').text(selectedCheckBoxCount);
            pointsDataTable.updateOptions({updateselectedData: selectedPointsData})
            changeNextButtonState();
        }

        function updatePointsMetaData()
        {
            if($(dom).find("#button-tab-unsecured-points").hasClass("active")){
                $(dom).find(".points-meta-data").show();
            }
            else{
                $(dom).find(".points-meta-data").hide();
            }
        }

        function searchUserInput() {
            //clear timeout when you decide to search
            userTypingTimeout = null;

            var tableWrapper,searchInput;
            if($(dom).find("#button-tab-unsecured-points").hasClass("active")){
                searchInput  = $(dom).find("#pdm-search-input").val().trim();
                tableWrapper = $(dom).find(".points-table-wrapper");
                if(searchInput && searchInput.length > 0) {
                    var regex = RegExp(searchInput, 'i');
                    var tempData = {};
                    tempData["columns"] = pointsData.columns.slice();
                    tempData["rows"] = pointsData.rows.slice();
                    tempData.rows = tempData.rows.filter(function (point) {
                        return regex.test(point["name"]);
                    });
                    pointsDataTable = datatable({
                        parent: tableWrapper[0],
                        tableData: tempData,
                        rowSelectionCallback: getSelectedData,
                        updateselectedData: selectedPointsData,
                        noDataMesage:lex.get("pdm.PointDatabaseManager.deletePointsView.nopoints.message")
                    });
                } else {
                    pointsDataTable = datatable({
                        parent: tableWrapper[0],
                        tableData: pointsData,
                        rowSelectionCallback: getSelectedData,
                        updateselectedData: selectedPointsData,
                        noDataMesage:lex.get("pdm.PointDatabaseManager.deletePointsView.nopoints.message")
                    });
                }
            }
            else{
                searchInput  = $(dom).find("#pdm-secure-search-input").val().trim();
                tableWrapper = $(dom).find(".secure-points-table-wrapper");
                if(searchInput && searchInput.length > 0) {
                    var regex = RegExp(searchInput, 'i');
                    var tempData = {};
                    tempData["columns"] = securePointsData.columns.slice();
                    tempData["rows"] = securePointsData.rows.slice();
                    tempData.rows = tempData.rows.filter(function (point) {
                        return regex.test(point["name"]);
                    });
                    securePointsDataTable = datatable({
                        parent: tableWrapper[0],
                        tableData: tempData,
                        rowSelectionCallback: attachRadioButtonEvent,
                        updateselectedData: selectedSecurePoint,
                        noDataMesage:lex.get("pdm.PointDatabaseManager.deletePointsView.nopoints.message")
                    });
                } else {
                    securePointsDataTable = datatable({
                        parent: tableWrapper[0],
                        tableData: securePointsData,
                        rowSelectionCallback: attachRadioButtonEvent,
                        updateselectedData: selectedSecurePoint,
                        noDataMesage:lex.get("pdm.PointDatabaseManager.deletePointsView.nopoints.message")
                    });
                }
            }
            attachRadioButtonEvent();
            updatePointsMetaData();
        }

        function makeDataSearchable() {
            $(dom).find("#pdm-search-input,#pdm-secure-search-input").on("keyup", function () {
                if (userTypingTimeout != null) {
                    clearTimeout(userTypingTimeout);
                }
                userTypingTimeout = setTimeout(searchUserInput, 150);
            });
        }

        function updateData(){
            baja.Ord.make("file:^edm/edm.xml").get({lease:true})
                .then(function(response){
                    var i, point, pointList, pointData = {"columns":columns, "rows":[]};
                    var securePointData = {"columns":columns, "rows":[]};
                    $.get(response.getReadUri()).then(function(points) {
                        pointList = $(points).find('Item');
                        if(pointList !== null && pointList.length > 0) {
                            for(i=0; i< pointList.length; i++) {
                                point = pointList[i];
                                if($(point).attr("status") && $(point).attr("status") === "1") {
                                    continue;
                                }
                                if($(point).attr("secure") && $(point).attr("secure") === "1") {
                                    securePointData.rows.push({
                                    name:$(point).attr("name"),
                                    path:$(point).attr("path"),
                                    handle:$(point).attr("handle"),
                                    delete:$(point).attr("delete")
                                 });
                                }
                                else{
                                   pointData.rows.push({
                                    name:$(point).attr("name"),
                                    path:$(point).attr("path"),
                                    handle:$(point).attr("handle"),
                                    delete:$(point).attr("delete")
                                   });
                              }
                            }
                        }
                        //once points data is ready - hide progress bar and display the table
                        //$(dom).find(".operation-progress").removeClass("show-progress");
                        var tableWrapper = $(dom).find(".points-table-wrapper");
                        pointsDataTable = datatable({
                            parent: tableWrapper[0],
                            tableData: pointData,
                            rowSelectionCallback: getSelectedData,
                            updateselectedData: selectedPointsData,
                            noDataMesage:lex.get("pdm.PointDatabaseManager.deletePointsView.nopoints.message")
                        });
                        pointsData = {};
                        pointsData["columns"] = pointData.columns.slice();
                        pointsData["rows"] = pointData.rows.slice();
                        $(dom).find(".total-count").html(pointData.rows.length);
                        pointsSize = pointData.rows.length;

                        var secureTableWrapper = $(dom).find(".secure-points-table-wrapper");
                        securePointsDataTable = datatable({
                            parent: secureTableWrapper[0],
                            tableData: securePointData,
                            rowSelectionCallback: attachRadioButtonEvent,
                            updateselectedData: selectedSecurePoint,
                            noDataMesage:lex.get("pdm.PointDatabaseManager.deletePointsView.nopoints.message")
                        });
                        securePointsData = {};
                        securePointsData["columns"] = securePointData.columns.slice();
                        securePointsData["rows"] = securePointData.rows.slice();

                        securePointsSize = securePointData.rows.length;
                        if(securePointsSize <= 0){
                            $(dom).find(".tabs").hide()
                            $(dom).find("#button-tab-secured-points").hide()}
                        else {
                            $(dom).find("#button-tab-secured-points").show()}
                        updatePointsMetaData();
                        reviewPointsView.prototype.attachButtonEvents();
                    });
                });
        }

        function changeNextButtonState() {
            //if at least one point is selected show action slider
            var step4 = ("#PDM-contents-step4-details");
            if((selectedPointsData && selectedPointsData.length > 0) || ($(step4).find("tr > td .ui-state-active").length > 0) || selectedSecurePoint !== null) {
                reviewPointsView.updateActionButtonsStatus(
                    $(dom).parent(),
                    {
                        disabledBack: false,
                        disabledNext: false
                    }
                );
                return true;
            } else {
                reviewPointsView.updateActionButtonsStatus(
                    $(dom).parent(),
                    {
                        disabledBack: false,
                        disabledNext: true
                    }
                );
                return false;
            }
        }

        function updateProgressBar (progress, success) {

            var bar;

            if (!dom) {
                return; // If an event comes in when we aren't ready for it, ignore it.
            }

            bar = dom.find('#wizard-progress-bar');

            if (progress > MIN_PROGRESS && progress < MAX_PROGRESS) {
                bar.animate({ width: progress + '%' });
            }
            else {
                bar.width(progress + '%'); // Don't animate when setting it back to zero, as it looks weird.
            }

            if (success !== undefined) {
                if (success) {
                    bar.removeClass(PROGRESS_BAR_RUNNING_CSS_CLASS + ' ' + PROGRESS_BAR_FAIL_CSS_CLASS)
                       .addClass(PROGRESS_BAR_SUCCESS_CSS_CLASS);
                }
                else {
                    bar.removeClass(PROGRESS_BAR_RUNNING_CSS_CLASS + ' ' + PROGRESS_BAR_SUCCESS_CSS_CLASS)
                       .addClass(PROGRESS_BAR_FAIL_CSS_CLASS);
                }
            }
            else {
                bar.removeClass(PROGRESS_BAR_FAIL_CSS_CLASS + ' ' + PROGRESS_BAR_SUCCESS_CSS_CLASS)
                   .addClass(PROGRESS_BAR_RUNNING_CSS_CLASS);
            }
        }

        function updateSelectedPoints(resolve) {
            baja.Ord.make("file:^edm/edm.xml").get({lease:true,
                ok: function(response) {
                    var i, z, point, pointList, networkList, xmlDoc = $.parseXML("<Items/>");
                    var xmlElements = xmlDoc.getElementsByTagName("Items")[0];

                    $.get(response.getReadUri(), function(points) {
                        networkList = $(points).find('Network');

                        if (networkList !== null && networkList.length > 0) {
                            for (z = 0; z < networkList.length; z++) {
                                pointList = $(networkList[z]).find('Item');
                                if (pointList !== null && pointList.length > 0) {
                                    for (i = 0; i < pointList.length; i++) {
                                        point = pointList[i];
                                        if($(point).attr("secure") && $(point).attr("secure") === "1") {
                                            if(selectedSecurePoint === $(point).attr('path')){
                                                $(point).attr("delete", "1");
                                            }
                                            else{
                                                $(point).attr("delete", "0");
                                            }
                                        }else {
                                            $(point).attr("delete", isSelectedPoint($(point).attr("handle"), selectedPointsData));
                                        }
                                        networkList[z].appendChild(point);
                                    }
                                }
                                xmlElements.appendChild(networkList[z]);
                            }
                            baja.Ord.make("file:^edm/edm.xml").get({
                                ok: function (res) {
                                    $.ajax({
                                        url: res.getWriteUri(),
                                        type: 'PUT',
                                        data: xmlElements.outerHTML,
                                        headers: {
                                            "x-niagara-csrfToken": csrfUtil.getCsrfToken()
                                        },
                                        success: function () {
                                            resolve(true);
                                        }
                                    });
                                }
                            });
                        }
                    });
                },
                fail: function(response) {
                // Failed
                }
            });
        }

        function isSelectedPoint(pointHandle, selectedPoints) {
            if(selectedPoints !== null && selectedPoints.length > 0 && pointHandle !== null) {
                var output = selectedPoints.includes(pointHandle);
                return output ? "1" : "0";
            }
            return "0";
        }

        return reviewPointsView;
    });
